@@ -9,6 +9,7 @@
  *
  */
 
+#include "version.h"
 #include "device-tree.h"
 #include "osutils.h"
 #include <sys/types.h>
@@ -18,8 +19,7 @@
 #include <unistd.h>
 #include <dirent.h>
 
-static const char rcsid[] =
-  "@(#) $Id: device-tree.cc 1319 2006-05-09 07:48:12Z ezix $";
+__ID("@(#) $Id: device-tree.cc 1907 2007-10-13 22:23:16Z lyonel $");
 
 #define DIMMINFOSIZE 0x80
 typedef __uint8_t dimminfo_buf[DIMMINFOSIZE];
@@ -48,8 +48,9 @@ static unsigned long get_long(const string & path)
   return result;
 }
 
+
 static vector < string > get_strings(const string & path,
-				     unsigned int offset = 0)
+unsigned int offset = 0)
 {
   vector < string > result;
   char *strings = NULL;
@@ -66,17 +67,17 @@ static vector < string > get_strings(const string & path,
       strings = (char *) malloc(buf.st_size + 1);
       if (strings)
       {
-	memset(strings, 0, buf.st_size + 1);
-	read(fd, strings, buf.st_size);
-	curstring = strings + offset;
+        memset(strings, 0, buf.st_size + 1);
+        read(fd, strings, buf.st_size);
+        curstring = strings + offset;
 
-	while (strlen(curstring))
-	{
-	  result.push_back(string(curstring));
-	  curstring += strlen(curstring) + 1;
-	}
+        while (strlen(curstring))
+        {
+          result.push_back(string(curstring));
+          curstring += strlen(curstring) + 1;
+        }
 
-	free(strings);
+        free(strings);
       }
     }
 
@@ -86,17 +87,19 @@ static vector < string > get_strings(const string & path,
   return result;
 }
 
+
 static void scan_devtree_root(hwNode & core)
 {
   core.setClock(get_long(DEVICETREE "/clock-frequency"));
 }
 
+
 static void scan_devtree_bootrom(hwNode & core)
 {
   if (exists(DEVICETREE "/rom/boot-rom"))
   {
     hwNode bootrom("firmware",
-		   hw::memory);
+      hw::memory);
     string upgrade = "";
     unsigned long base = 0;
     unsigned long size = 0;
@@ -107,7 +110,7 @@ static void scan_devtree_bootrom(hwNode & core)
       setVersion(get_string(DEVICETREE "/rom/boot-rom/BootROM-version"));
 
     if ((upgrade =
-	 get_string(DEVICETREE "/rom/boot-rom/write-characteristic")) != "")
+      get_string(DEVICETREE "/rom/boot-rom/write-characteristic")) != "")
     {
       bootrom.addCapability("upgrade");
       bootrom.addCapability(upgrade);
@@ -125,27 +128,28 @@ static void scan_devtree_bootrom(hwNode & core)
     }
 
     bootrom.claim();
-    //bootrom.setLogicalName(DEVICETREE "/rom");
+//bootrom.setLogicalName(DEVICETREE "/rom");
     core.addChild(bootrom);
   }
 
   if (exists(DEVICETREE "/openprom"))
   {
     hwNode openprom("firmware",
-		    hw::memory);
+      hw::memory);
 
     openprom.setProduct(get_string(DEVICETREE "/openprom/model"));
 
     if (exists(DEVICETREE "/openprom/supports-bootinfo"))
       openprom.addCapability("bootinfo");
 
-    //openprom.setLogicalName(DEVICETREE "/openprom");
+//openprom.setLogicalName(DEVICETREE "/openprom");
     openprom.setLogicalName(DEVICETREE);
     openprom.claim();
     core.addChild(openprom);
   }
 }
 
+
 static string cpubusinfo(int cpu)
 {
   char buffer[20];
@@ -155,6 +159,7 @@ static string cpubusinfo(int cpu)
   return string(buffer);
 }
 
+
 static void scan_devtree_cpu(hwNode & core)
 {
   struct dirent **namelist;
@@ -171,15 +176,15 @@ static void scan_devtree_cpu(hwNode & core)
     for (int i = 0; i < n; i++)
     {
       string basepath =
-	string(DEVICETREE "/cpus/") + string(namelist[i]->d_name);
+        string(DEVICETREE "/cpus/") + string(namelist[i]->d_name);
       unsigned long version = 0;
       hwNode cpu("cpu",
-		 hw::processor);
+        hw::processor);
       struct dirent **cachelist;
       int ncache;
 
       if (hw::strip(get_string(basepath + "/device_type")) != "cpu")
-	break;			// oops, not a CPU!
+        break;                                    // oops, not a CPU!
 
       cpu.setProduct(get_string(basepath + "/name"));
       cpu.setDescription("CPU");
@@ -188,35 +193,35 @@ static void scan_devtree_cpu(hwNode & core)
       cpu.setSize(get_long(basepath + "/clock-frequency"));
       cpu.setClock(get_long(basepath + "/bus-frequency"));
       if (exists(basepath + "/altivec"))
-	cpu.addCapability("altivec");
+        cpu.addCapability("altivec");
 
       version = get_long(basepath + "/cpu-version");
       if (version != 0)
       {
-	int minor = version & 0x00ff;
-	int major = (version & 0xff00) >> 8;
-	char buffer[20];
+        int minor = version & 0x00ff;
+        int major = (version & 0xff00) >> 8;
+        char buffer[20];
 
-	snprintf(buffer, sizeof(buffer), "%lx.%d.%d",
-		 (version & 0xffff0000) >> 16, major, minor);
-	cpu.setVersion(buffer);
+        snprintf(buffer, sizeof(buffer), "%lx.%d.%d",
+          (version & 0xffff0000) >> 16, major, minor);
+        cpu.setVersion(buffer);
 
       }
       if (hw::strip(get_string(basepath + "/state")) != "running")
-	cpu.disable();
+        cpu.disable();
 
       if (exists(basepath + "/performance-monitor"))
-	cpu.addCapability("performance-monitor");
+        cpu.addCapability("performance-monitor");
 
       if (exists(basepath + "/d-cache-size"))
       {
-	hwNode cache("cache",
-		     hw::memory);
+        hwNode cache("cache",
+          hw::memory);
 
-	cache.setDescription("L1 Cache");
-	cache.setSize(get_long(basepath + "/d-cache-size"));
-	if (cache.getSize() > 0)
-	  cpu.addChild(cache);
+        cache.setDescription("L1 Cache");
+        cache.setSize(get_long(basepath + "/d-cache-size"));
+        if (cache.getSize() > 0)
+          cpu.addChild(cache);
       }
 
       pushd(basepath);
@@ -224,39 +229,39 @@ static void scan_devtree_cpu(hwNode & core)
       popd();
       if (ncache > 0)
       {
-	for (int j = 0; j < ncache; j++)
-	{
-	  hwNode cache("cache",
-		       hw::memory);
-	  string cachebase = basepath + "/" + cachelist[j]->d_name;
+        for (int j = 0; j < ncache; j++)
+        {
+          hwNode cache("cache",
+            hw::memory);
+          string cachebase = basepath + "/" + cachelist[j]->d_name;
 
-	  if (hw::strip(get_string(cachebase + "/device_type")) != "cache" &&
-	      hw::strip(get_string(cachebase + "/device_type")) != "l2-cache")
-	    break;		// oops, not a cache!
+          if (hw::strip(get_string(cachebase + "/device_type")) != "cache" &&
+            hw::strip(get_string(cachebase + "/device_type")) != "l2-cache")
+            break;                                // oops, not a cache!
 
-	  cache.setDescription("L2 Cache");
-	  cache.setSize(get_long(cachebase + "/d-cache-size"));
-	  cache.setClock(get_long(cachebase + "/clock-frequency"));
+          cache.setDescription("L2 Cache");
+          cache.setSize(get_long(cachebase + "/d-cache-size"));
+          cache.setClock(get_long(cachebase + "/clock-frequency"));
 
-	  if (exists(cachebase + "/cache-unified"))
-	    cache.setDescription(cache.getDescription() + " (unified)");
-	  else
-	  {
-	    hwNode icache = cache;
-	    cache.setDescription(cache.getDescription() + " (data)");
-	    icache.setDescription(icache.getDescription() + " (instruction)");
-	    icache.setSize(get_long(cachebase + "/i-cache-size"));
+          if (exists(cachebase + "/cache-unified"))
+            cache.setDescription(cache.getDescription() + " (unified)");
+          else
+          {
+            hwNode icache = cache;
+            cache.setDescription(cache.getDescription() + " (data)");
+            icache.setDescription(icache.getDescription() + " (instruction)");
+            icache.setSize(get_long(cachebase + "/i-cache-size"));
 
-	    if (icache.getSize() > 0)
-	      cpu.addChild(icache);
-	  }
+            if (icache.getSize() > 0)
+              cpu.addChild(icache);
+          }
 
-	  if (cache.getSize() > 0)
-	    cpu.addChild(cache);
+          if (cache.getSize() > 0)
+            cpu.addChild(cache);
 
-	  free(cachelist[j]);
-	}
-	free(cachelist);
+          free(cachelist[j]);
+        }
+        free(cachelist);
       }
 
       core.addChild(cpu);
@@ -267,9 +272,10 @@ static void scan_devtree_cpu(hwNode & core)
   }
 }
 
+
 static void scan_devtree_memory(hwNode & core)
 {
-  int currentmc = -1;		// current memory controller
+  int currentmc = -1;                             // current memory controller
   hwNode *memory = core.getChild("memory");
 
   while (true)
@@ -298,11 +304,11 @@ static void scan_devtree_memory(hwNode & core)
     {
       if (currentmc < 0)
       {
-	currentmc++;
-	continue;
+        currentmc++;
+        continue;
       }
       else
-	break;
+        break;
     }
 
     if (!memory || (currentmc > 0))
@@ -317,52 +323,52 @@ static void scan_devtree_memory(hwNode & core)
 
       if (fd2 >= 0)
       {
-	for (unsigned int i = 0; i < slotnames.size(); i++)
-	{
-	  unsigned long base = 0;
-	  unsigned long size = 0;
-	  hwNode bank("bank",
-		      hw::memory);
+        for (unsigned int i = 0; i < slotnames.size(); i++)
+        {
+          unsigned long base = 0;
+          unsigned long size = 0;
+          hwNode bank("bank",
+            hw::memory);
 
-	  read(fd2, &base, sizeof(base));
-	  read(fd2, &size, sizeof(size));
+          read(fd2, &base, sizeof(base));
+          read(fd2, &size, sizeof(size));
 
-	  if (fd >= 0)
-	  {
-	    dimminfo_buf dimminfo;
+          if (fd >= 0)
+          {
+            dimminfo_buf dimminfo;
 
-	    if (read(fd, &dimminfo, sizeof(dimminfo)) > 0)
-	    {
-	      if (size > 0)
-	      {
-		char dimmversion[20];
-		snprintf(dimmversion, sizeof(dimmversion),
-			 "%02X%02X,%02X %02X,%02X", dimminfo[0x5b],
-			 dimminfo[0x5c], dimminfo[0x5d], dimminfo[0x5e],
-			 dimminfo[0x48]);
-		bank.setSerial(string((char *) &dimminfo + 0x49, 18));
-		bank.setVersion(dimmversion);
-	      }
-	    }
-	  }
+            if (read(fd, &dimminfo, sizeof(dimminfo)) > 0)
+            {
+              if (size > 0)
+              {
+                char dimmversion[20];
+                snprintf(dimmversion, sizeof(dimmversion),
+                  "%02X%02X,%02X %02X,%02X", dimminfo[0x5b],
+                  dimminfo[0x5c], dimminfo[0x5d], dimminfo[0x5e],
+                  dimminfo[0x48]);
+                bank.setSerial(string((char *) &dimminfo + 0x49, 18));
+                bank.setVersion(dimmversion);
+              }
+            }
+          }
 
           if(size>0)
             bank.addHint("icon", string("memory"));
-	  bank.setDescription("Memory bank");
-	  bank.setSlot(slotnames[i]);
-	  //bank.setPhysId(base);
-	  if (i < dimmtypes.size())
-	    bank.setDescription(dimmtypes[i]);
-	  if (i < dimmspeeds.size())
-	    bank.setProduct(hw::strip(dimmspeeds[i]));
-	  bank.setSize(size);
-	  memory->addChild(bank);
-	}
-	close(fd2);
+          bank.setDescription("Memory bank");
+          bank.setSlot(slotnames[i]);
+//bank.setPhysId(base);
+          if (i < dimmtypes.size())
+            bank.setDescription(dimmtypes[i]);
+          if (i < dimmspeeds.size())
+            bank.setProduct(hw::strip(dimmspeeds[i]));
+          bank.setSize(size);
+          memory->addChild(bank);
+        }
+        close(fd2);
       }
 
       if (fd >= 0)
-	close(fd);
+        close(fd);
       currentmc++;
     }
     else
@@ -372,6 +378,7 @@ static void scan_devtree_memory(hwNode & core)
   }
 }
 
+
 struct pmac_mb_def
 {
   const char *model;
@@ -379,11 +386,12 @@ struct pmac_mb_def
   const char *icon;
 };
 
-static struct pmac_mb_def pmac_mb_defs[] = {
-  /*
-   * Warning: ordering is important as some models may claim
-   * * beeing compatible with several types 
-   */
+static struct pmac_mb_def pmac_mb_defs[] =
+{
+/*
+ * Warning: ordering is important as some models may claim
+ * * beeing compatible with several types
+ */
   {"AAPL,8500", "PowerMac 8500/8600", ""},
   {"AAPL,9500", "PowerMac 9500/9600", ""},
   {"AAPL,7200", "PowerMac 7200", ""},
@@ -417,25 +425,40 @@ static struct pmac_mb_def pmac_mb_defs[] = {
   {"PowerMac6,1", "iMac LCD 17\"", ""},
   {"PowerMac7,2", "PowerMac G5", "powermacg5"},
   {"PowerMac7,3", "PowerMac G5", "powermacg5"},
+  {"PowerMac8,1", "iMac G5", ""},
+  {"PowerMac8,2", "iMac G5", ""},
   {"PowerMac10,1", "Mac mini", "mini"},
-  {"PowerBook1,1", "PowerBook 101 (Lombard)", ""},
-  {"PowerBook2,1", "iBook (first generation)", ""},
-  {"PowerBook2,2", "iBook FireWire", ""},
-  {"PowerBook3,1", "PowerBook Pismo", ""},
-  {"PowerBook3,2", "PowerBook Titanium", ""},
-  {"PowerBook3,3", "PowerBook Titanium w/ Gigabit Ethernet", ""},
-  {"PowerBook3,4", "PowerBook Titanium w/ DVI", ""},
-  {"PowerBook3,5", "PowerBook Titanium 1GHz", ""},
-  {"PowerBook4,1", "iBook 12\" (May 2001)", ""},
-  {"PowerBook4,2", "iBook 2", ""},
-  {"PowerBook4,3", "iBook 2 rev. 2 (Nov 2002)", ""},
-  {"PowerBook4,4", "iBook 2 rev. 2", ""},
-  {"PowerBook5,1", "PowerBook G4 17\"", ""},
-  {"PowerBook5,2", "PowerBook G4 15\"", ""},
-  {"PowerBook5,3", "PowerBook G4 17\" rev. 2", ""},
-  {"PowerBook6,1", "PowerBook G4 12\"", ""},
-  {"PowerBook6,2", "PowerBook G4 12\"", ""},
-  {"PowerBook6,3", "iBook G4", ""},
+  {"PowerMac10,2", "Mac mini", "mini"},
+  {"PowerMac11,2", "PowerMac G5", "powermacg5"},
+  {"PowerMac12,1", "iMac G5", ""},
+  {"PowerBook1,1", "PowerBook 101 (Lombard)", "laptop"},
+  {"PowerBook2,1", "iBook (first generation)", "laptop"},
+  {"PowerBook2,2", "iBook FireWire", "laptop"},
+  {"PowerBook3,1", "PowerBook Pismo", "laptop"},
+  {"PowerBook3,2", "PowerBook Titanium", "laptop"},
+  {"PowerBook3,3", "PowerBook Titanium w/ Gigabit Ethernet", "laptop"},
+  {"PowerBook3,4", "PowerBook Titanium w/ DVI", "laptop"},
+  {"PowerBook3,5", "PowerBook Titanium 1GHz", "laptop"},
+  {"PowerBook4,1", "iBook 12\" (May 2001)", "laptop"},
+  {"PowerBook4,2", "iBook 2", "laptop"},
+  {"PowerBook4,3", "iBook 2 rev. 2 (Nov 2002)", "laptop"},
+  {"PowerBook4,4", "iBook 2 rev. 2", "laptop"},
+  {"PowerBook5,1", "PowerBook G4 17\"", "laptop"},
+  {"PowerBook5,2", "PowerBook G4 15\"", "laptop"},
+  {"PowerBook5,3", "PowerBook G4 17\" 1.33GHz", "laptop"},
+  {"PowerBook5,4", "PowerBook G4 15\" 1.5/1.33GHz", "laptop"},
+  {"PowerBook5,5", "PowerBook G4 17\" 1.5GHz", "laptop"},
+  {"PowerBook5,6", "PowerBook G4 15\" 1.67/1.5GHz", "laptop"},
+  {"PowerBook5,7", "PowerBook G4 17\" 1.67GHz", "laptop"},
+  {"PowerBook5,8", "PowerBook G4 15\" double layer SD", "laptop"},
+  {"PowerBook5,9", "PowerBook G4 17\" double layer SD", "laptop"},
+  {"PowerBook6,1", "PowerBook G4 12\"", "laptop"},
+  {"PowerBook6,2", "PowerBook G4 12\" DVI", "laptop"},
+  {"PowerBook6,3", "iBook G4", "laptop"},
+  {"PowerBook6,4", "PowerBook G4 12\"", "laptop"},
+  {"PowerBook6,5", "iBook G4", "laptop"},
+  {"PowerBook6,7", "iBook G4", "laptop"},
+  {"PowerBook6,8", "PowerBook G4 12\" 1.5GHz", "laptop"},
   {"RackMac1,1", "XServe", ""},
   {"RackMac1,2", "XServe rev. 2", ""},
   {"RackMac3,1", "XServe G5", ""},
@@ -448,25 +471,27 @@ static bool get_apple_model(hwNode & n)
     return false;
 
   for (unsigned int i = 0; i < sizeof(pmac_mb_defs) / sizeof(pmac_mb_def);
-       i++)
-    if (model == pmac_mb_defs[i].model)
-    {
-      n.setProduct(pmac_mb_defs[i].modelname);
-      n.addHint("icon", string(pmac_mb_defs[i].icon));
-    }
+    i++)
+  if (model == pmac_mb_defs[i].model)
+  {
+    n.setProduct(pmac_mb_defs[i].modelname);
+    n.addHint("icon", string(pmac_mb_defs[i].icon));
+  }
 
   return false;
 }
 
+
 static void fix_serial_number(hwNode & n)
 {
   string serial = n.getSerial();
 
-  if(serial.find('\0')==string::npos) return; // nothing to do
+  if(serial.find('\0')==string::npos) return;     // nothing to do
 
   n.setSerial(hw::strip(serial.substr(13)) + hw::strip(serial.substr(0,13)));
 }
 
+
 bool scan_device_tree(hwNode & n)
 {
   hwNode *core = n.getChild("core");
