@@ -1,14 +1,17 @@
 #include "version.h"
 #include "disk.h"
+#include "osutils.h"
+#include "heuristics.h"
 #include "partitions.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <unistd.h>
+
 //#include <linux/fs.h>
 
-__ID("@(#) $Id: disk.cc 1726 2007-02-13 21:38:11Z lyonel $");
+__ID("@(#) $Id: disk.cc 1907 2007-10-13 22:23:16Z lyonel $");
 
 #ifndef BLKROGET
 #define BLKROGET   _IO(0x12,94)                   /* get read-only status (0 = read_write) */
