@@ -1,17 +1,21 @@
+#define _XOPEN_SOURCE
 #include "hw.h"
 #include "osutils.h"
 #include "version.h"
 #include "options.h"
+#include "heuristics.h"
 #include <vector>
 #include <map>
 #include <sstream>
 #include <unistd.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <unistd.h>
+#include <sys/utsname.h>
 
 using namespace hw;
 
-static const char rcsid[] = "@(#) $Id: hw.cc 1319 2006-05-09 07:48:12Z ezix $";
+__ID("@(#) $Id: hw.cc 1907 2007-10-13 22:23:16Z lyonel $");
 
 struct hwNode_i
 {
@@ -25,34 +29,45 @@ struct hwNode_i
   unsigned long long capacity;
   unsigned long long clock;
   unsigned int width;
-    vector < hwNode > children;
-    vector < string > attracted;
-    vector < string > features;
-    vector < string > logicalnames;
-    map < string, string > features_descriptions;
-    vector < resource > resources;
-    map < string, string > config;
-    map < string, value > hints;
+  vector < hwNode > children;
+  vector < string > attracted;
+  vector < string > features;
+  vector < string > logicalnames;
+  map < string, string > features_descriptions;
+  vector < resource > resources;
+  map < string, string > config;
+  map < string, value > hints;
 };
 
 string hw::strip(const string & s)
 {
   string result = s;
 
-  while ((result.length() > 0) && (result[0] <= ' '))
+  while ((result.length() > 0) && ((uint8_t)result[0] <= ' '))
     result.erase(0, 1);
-  while ((result.length() > 0) && (result[result.length() - 1] <= ' '))
-    result.erase(result.length() - 1, 1);
+  while ((result.length() > 0) && ((uint8_t)result[result.length() - 1] <= ' '))
+    result.erase(result.length() - 1);
+
   for (unsigned int i = 0; i < result.length(); i++)
-    if (result[0] <= ' ')
+    if ((uint8_t)result[i] < ' ')
     {
       result.erase(i, 1);
       i--;
     }
 
+  result = utf8_sanitize(result);
+
   return result;
 }
 
+string hw::asString(long n)
+{
+  char buffer[256];
+
+  snprintf(buffer, sizeof(buffer), "%ld", n);
+  return string(buffer);
+}
+
 static string cleanupId(const string & id)
 {
   string result = strip(id);
@@ -67,10 +82,11 @@ static string cleanupId(const string & id)
   return result;
 }
 
+
 hwNode::hwNode(const string & id,
-	       hwClass c,
-	       const string & vendor,
-	       const string & product, const string & version)
+hwClass c,
+const string & vendor,
+const string & product, const string & version)
 {
   This = NULL;
   This = new hwNode_i;
@@ -97,6 +113,7 @@ hwNode::hwNode(const string & id,
   This->dev = string("");
 }
 
+
 hwNode::hwNode(const hwNode & o)
 {
   This = NULL;
@@ -109,16 +126,18 @@ hwNode::hwNode(const hwNode & o)
     (*This) = (*o.This);
 }
 
+
 hwNode::~hwNode()
 {
   if (This)
     delete This;
 }
 
+
 hwNode & hwNode::operator = (const hwNode & o)
 {
   if (this == &o)
-    return *this;		// self-affectation
+    return *this;                                 // self-affectation
 
   if (This)
     delete This;
@@ -134,6 +153,7 @@ hwNode & hwNode::operator = (const hwNode & o)
   return *this;
 }
 
+
 hwClass hwNode::getClass() const
 {
   if (This)
@@ -142,68 +162,73 @@ hwClass hwNode::getClass() const
     return hw::generic;
 }
 
+
 const char *hwNode::getClassName() const
 {
   if (This)
   {
     switch (This->deviceclass)
     {
-    case processor:
-      return "processor";
+      case processor:
+        return "processor";
 
-    case memory:
-      return "memory";
+      case memory:
+        return "memory";
 
-    case address:
-      return "address";
+      case address:
+        return "address";
 
-    case storage:
-      return "storage";
+      case storage:
+        return "storage";
 
-    case disk:
-      return "disk";
+      case disk:
+        return "disk";
 
-    case tape:
-      return "tape";
+      case tape:
+        return "tape";
 
-    case hw::system:
-      return "system";
+      case hw::system:
+        return "system";
 
-    case bridge:
-      return "bridge";
+      case bridge:
+        return "bridge";
 
-    case bus:
-      return "bus";
+      case bus:
+        return "bus";
 
-    case network:
-      return "network";
+      case network:
+        return "network";
 
-    case display:
-      return "display";
+      case display:
+        return "display";
 
-    case input:
-      return "input";
+      case input:
+        return "input";
 
-    case printer:
-      return "printer";
+      case printer:
+        return "printer";
 
-    case multimedia:
-      return "multimedia";
+      case multimedia:
+        return "multimedia";
 
-    case communication:
-      return "communication";
+      case communication:
+        return "communication";
 
-    case power:
-      return "power";
+      case power:
+        return "power";
 
-    default:
-      return "generic";
+      case volume:
+        return "volume";
+
+      default:
+        return "generic";
     }
   }
   else
     return "generic";
 }
 
+
 void hwNode::setClass(hwClass c)
 {
   if (!This)
@@ -212,6 +237,7 @@ void hwNode::setClass(hwClass c)
   This->deviceclass = c;
 }
 
+
 bool hwNode::enabled() const
 {
   if (!This)
@@ -220,6 +246,7 @@ bool hwNode::enabled() const
   return (This->enabled);
 }
 
+
 bool hwNode::disabled() const
 {
   if (!This)
@@ -228,6 +255,7 @@ bool hwNode::disabled() const
   return !(This->enabled);
 }
 
+
 void hwNode::enable()
 {
   if (!This)
@@ -236,6 +264,7 @@ void hwNode::enable()
   This->enabled = true;
 }
 
+
 void hwNode::disable()
 {
   if (!This)
@@ -244,6 +273,7 @@ void hwNode::disable()
   This->enabled = false;
 }
 
+
 bool hwNode::claimed() const
 {
   if (!This)
@@ -254,14 +284,15 @@ bool hwNode::claimed() const
 
   for (unsigned int i = 0; i < This->children.size(); i++)
     if (This->children[i].claimed())
-    {
-      This->claimed = true;
-      return true;
-    }
+  {
+    This->claimed = true;
+    return true;
+  }
 
   return false;
 }
 
+
 void hwNode::claim(bool claimchildren)
 {
   if (!This)
@@ -276,6 +307,7 @@ void hwNode::claim(bool claimchildren)
     This->children[i].claim(claimchildren);
 }
 
+
 void hwNode::unclaim()
 {
   if (!This)
@@ -284,6 +316,7 @@ void hwNode::unclaim()
   This->claimed = false;
 }
 
+
 string hwNode::getId() const
 {
   if (This)
@@ -292,6 +325,7 @@ string hwNode::getId() const
     return "";
 }
 
+
 void hwNode::setId(const string & id)
 {
   if (!This)
@@ -300,6 +334,7 @@ void hwNode::setId(const string & id)
   This->id = cleanupId(id);
 }
 
+
 void hwNode::setHandle(const string & handle)
 {
   if (!This)
@@ -308,6 +343,7 @@ void hwNode::setHandle(const string & handle)
   This->handle = handle;
 }
 
+
 string hwNode::getHandle() const
 {
   if (This)
@@ -316,6 +352,7 @@ string hwNode::getHandle() const
     return "";
 }
 
+
 string hwNode::getDescription() const
 {
   if (This)
@@ -324,12 +361,14 @@ string hwNode::getDescription() const
     return "";
 }
 
+
 void hwNode::setDescription(const string & description)
 {
   if (This)
     This->description = strip(description);
 }
 
+
 string hwNode::getVendor() const
 {
   if (This)
@@ -338,12 +377,14 @@ string hwNode::getVendor() const
     return "";
 }
 
+
 void hwNode::setVendor(const string & vendor)
 {
   if (This)
     This->vendor = strip(vendor);
 }
 
+
 string hwNode::getProduct() const
 {
   if (This)
@@ -352,12 +393,14 @@ string hwNode::getProduct() const
     return "";
 }
 
+
 void hwNode::setProduct(const string & product)
 {
   if (This)
     This->product = strip(product);
 }
 
+
 string hwNode::getVersion() const
 {
   if (This)
@@ -366,12 +409,14 @@ string hwNode::getVersion() const
     return "";
 }
 
+
 void hwNode::setVersion(const string & version)
 {
   if (This)
     This->version = strip(version);
 }
 
+
 string hwNode::getSerial() const
 {
   if (This)
@@ -380,12 +425,17 @@ string hwNode::getSerial() const
     return "";
 }
 
+
 void hwNode::setSerial(const string & serial)
 {
+  if(serial == "00000000-0000-0000-0000-000000000000")
+    return;
+
   if (This)
     This->serial = strip(serial);
 }
 
+
 string hwNode::getSlot() const
 {
   if (This)
@@ -394,12 +444,14 @@ string hwNode::getSlot() const
     return "";
 }
 
+
 void hwNode::setSlot(const string & slot)
 {
   if (This)
     This->slot = strip(slot);
 }
 
+
 unsigned long long hwNode::getStart() const
 {
   if (This)
@@ -408,12 +460,14 @@ unsigned long long hwNode::getStart() const
     return 0;
 }
 
+
 void hwNode::setStart(unsigned long long start)
 {
   if (This)
     This->start = start;
 }
 
+
 unsigned long long hwNode::getSize() const
 {
   if (This)
@@ -422,12 +476,14 @@ unsigned long long hwNode::getSize() const
     return 0;
 }
 
+
 void hwNode::setSize(unsigned long long size)
 {
   if (This)
     This->size = size;
 }
 
+
 unsigned long long hwNode::getCapacity() const
 {
   if (This)
@@ -436,12 +492,14 @@ unsigned long long hwNode::getCapacity() const
     return 0;
 }
 
+
 void hwNode::setCapacity(unsigned long long capacity)
 {
   if (This)
     This->capacity = capacity;
 }
 
+
 unsigned long long hwNode::getClock() const
 {
   if (This)
@@ -450,12 +508,14 @@ unsigned long long hwNode::getClock() const
     return 0;
 }
 
+
 void hwNode::setClock(unsigned long long clock)
 {
   if (This)
     This->clock = clock;
 }
 
+
 unsigned int hwNode::countChildren(hw::hwClass c) const
 {
   unsigned int count = 0;
@@ -473,6 +533,7 @@ unsigned int hwNode::countChildren(hw::hwClass c) const
   return count;
 }
 
+
 hwNode *hwNode::getChild(unsigned int i)
 {
   if (!This)
@@ -484,6 +545,7 @@ hwNode *hwNode::getChild(unsigned int i)
     return &(This->children[i]);
 }
 
+
 hwNode *hwNode::getChildByPhysId(const string & physid)
 {
   if (physid == "" || !This)
@@ -498,6 +560,7 @@ hwNode *hwNode::getChildByPhysId(const string & physid)
   return NULL;
 }
 
+
 hwNode *hwNode::getChildByPhysId(long physid)
 {
   char buffer[20];
@@ -515,6 +578,7 @@ hwNode *hwNode::getChildByPhysId(long physid)
   return NULL;
 }
 
+
 hwNode *hwNode::getChild(const string & id)
 {
   string baseid = id, path = "";
@@ -533,15 +597,16 @@ hwNode *hwNode::getChild(const string & id)
 
   for (unsigned int i = 0; i < This->children.size(); i++)
     if (This->children[i].getId() == baseid)
-    {
-      if (path == "")
-	return &(This->children[i]);
-      else
-	return This->children[i].getChild(path);
-    }
+  {
+    if (path == "")
+      return &(This->children[i]);
+    else
+      return This->children[i].getChild(path);
+  }
   return NULL;
 }
 
+
 hwNode *hwNode::findChild(bool(*matchfunction) (const hwNode &))
 {
   if (!This)
@@ -561,6 +626,7 @@ hwNode *hwNode::findChild(bool(*matchfunction) (const hwNode &))
   return NULL;
 }
 
+
 hwNode *hwNode::findChildByHandle(const string & handle)
 {
   if (!This)
@@ -580,6 +646,7 @@ hwNode *hwNode::findChildByHandle(const string & handle)
   return NULL;
 }
 
+
 hwNode *hwNode::findChildByLogicalName(const string & name)
 {
   unsigned int i = 0;
@@ -602,6 +669,7 @@ hwNode *hwNode::findChildByLogicalName(const string & name)
   return NULL;
 }
 
+
 hwNode *hwNode::findChildByBusInfo(const string & businfo)
 {
   if (!This)
@@ -624,6 +692,7 @@ hwNode *hwNode::findChildByBusInfo(const string & businfo)
   return NULL;
 }
 
+
 hwNode *hwNode::findChildByResource(const hw::resource & r)
 {
   if (!This)
@@ -643,8 +712,9 @@ hwNode *hwNode::findChildByResource(const hw::resource & r)
   return NULL;
 }
 
+
 static string generateId(const string & radical,
-			 int count)
+int count)
 {
   char buffer[10];
 
@@ -653,6 +723,7 @@ static string generateId(const string & radical,
   return radical + ":" + string(buffer);
 }
 
+
 hwNode *hwNode::addChild(const hwNode & node)
 {
   hwNode *existing = NULL;
@@ -663,13 +734,13 @@ hwNode *hwNode::addChild(const hwNode & node)
   if (!This)
     return NULL;
 
-  // first see if the new node is attracted by one of our children
+// first see if the new node is attracted by one of our children
   for (unsigned int i = 0; i < This->children.size(); i++)
     if (This->children[i].attractsNode(node))
       return This->children[i].addChild(node);
 
-  // find if another child already has the same physical id
-  // in that case, we remove BOTH physical ids and let auto-allocation proceed
+// find if another child already has the same physical id
+// in that case, we remove BOTH physical ids and let auto-allocation proceed
   if (node.getPhysId() != "")
     samephysid = getChildByPhysId(node.getPhysId());
   if (samephysid)
@@ -678,12 +749,12 @@ hwNode *hwNode::addChild(const hwNode & node)
   }
 
   existing = getChild(id);
-  if (existing)			// first rename existing instance
+  if (existing)                                   // first rename existing instance
   {
-    while (getChild(generateId(id, count)))	// find a usable name
+    while (getChild(generateId(id, count)))       // find a usable name
       count++;
 
-    existing->setId(generateId(id, count));	// rename
+    existing->setId(generateId(id, count));       // rename
   }
 
   while (getChild(generateId(id, count)))
@@ -697,9 +768,10 @@ hwNode *hwNode::addChild(const hwNode & node)
     This->children.back().setPhysId("");
 
   return &(This->children.back());
-  //return getChild(This->children.back().getId());
+//return getChild(This->children.back().getId());
 }
 
+
 void hwNode::attractHandle(const string & handle)
 {
   if (!This)
@@ -708,6 +780,7 @@ void hwNode::attractHandle(const string & handle)
   This->attracted.push_back(handle);
 }
 
+
 bool hwNode::attractsHandle(const string & handle) const
 {
   unsigned int i = 0;
@@ -725,6 +798,7 @@ bool hwNode::attractsHandle(const string & handle) const
   return false;
 }
 
+
 bool hwNode::attractsNode(const hwNode & node) const
 {
   if (!This || !node.This)
@@ -733,6 +807,7 @@ bool hwNode::attractsNode(const hwNode & node) const
   return attractsHandle(node.This->handle);
 }
 
+
 bool hwNode::isCapable(const string & feature) const
 {
   string featureid = cleanupId(feature);
@@ -747,8 +822,9 @@ bool hwNode::isCapable(const string & feature) const
   return false;
 }
 
+
 void hwNode::addCapability(const string & feature,
-			   const string & description)
+const string & description)
 {
   string features = feature;
 
@@ -765,21 +841,22 @@ void hwNode::addCapability(const string & feature,
     if (pos == string::npos)
     {
       if (!isCapable(cleanupId(features)))
-	This->features.push_back(cleanupId(features));
+        This->features.push_back(cleanupId(features));
       features = "";
     }
     else
     {
       string featureid = cleanupId(features.substr(0, pos));
       if (!isCapable(featureid))
-	This->features.push_back(featureid);
+        This->features.push_back(featureid);
       features = features.substr(pos + 1);
     }
   }
 }
 
+
 void hwNode::describeCapability(const string & feature,
-				const string & description)
+const string & description)
 {
   if (!This)
     return;
@@ -787,10 +864,10 @@ void hwNode::describeCapability(const string & feature,
   if (!isCapable(feature))
     return;
 
-
   This->features_descriptions[cleanupId(feature)] = strip(description);
 }
 
+
 string hwNode::getCapabilities() const
 {
   string result = "";
@@ -804,6 +881,7 @@ string hwNode::getCapabilities() const
   return strip(result);
 }
 
+
 string hwNode::getCapabilityDescription(const string & feature) const
 {
   string featureid = cleanupId(feature);
@@ -815,14 +893,15 @@ string hwNode::getCapabilityDescription(const string & feature) const
     return "";
 
   if (This->features_descriptions.find(featureid) ==
-      This->features_descriptions.end())
+    This->features_descriptions.end())
     return "";
 
   return This->features_descriptions[featureid];
 }
 
+
 void hwNode::setConfig(const string & key,
-		       const string & value)
+const string & value)
 {
   if (!This)
     return;
@@ -832,8 +911,9 @@ void hwNode::setConfig(const string & key,
     This->config.erase(This->config.find(key));
 }
 
+
 void hwNode::setConfig(const string & key,
-		       unsigned long long value)
+unsigned long long value)
 {
   if (!This)
     return;
@@ -841,6 +921,7 @@ void hwNode::setConfig(const string & key,
   This->config[key] = tostring(value);
 }
 
+
 string hwNode::getConfig(const string & key) const
 {
   if (!This)
@@ -852,6 +933,21 @@ string hwNode::getConfig(const string & key) const
   return This->config[key];
 }
 
+
+vector < string > hwNode::getConfigKeys() const
+{
+  vector < string > result;
+
+  if (!This)
+    return result;
+
+  for (map < string, string >::iterator i = This->config.begin();
+    i != This->config.end(); i++)
+  result.push_back(i->first);
+
+  return result;
+}
+
 vector < string > hwNode::getConfigValues(const string & separator) const
 {
   vector < string > result;
@@ -860,12 +956,13 @@ vector < string > hwNode::getConfigValues(const string & separator) const
     return result;
 
   for (map < string, string >::iterator i = This->config.begin();
-       i != This->config.end(); i++)
-    result.push_back(i->first + separator + i->second);
+    i != This->config.end(); i++)
+  result.push_back(i->first + separator + i->second);
 
   return result;
 }
 
+
 string hwNode::getLogicalName() const
 {
   if (This && (This->logicalnames.size()>0))
@@ -874,6 +971,7 @@ string hwNode::getLogicalName() const
     return "";
 }
 
+
 vector<string> hwNode::getLogicalNames() const
 {
   if (This)
@@ -882,24 +980,29 @@ vector<string> hwNode::getLogicalNames() const
     return vector<string>();
 }
 
+
 void hwNode::setLogicalName(const string & name)
 {
+  string n = strip(name);
+
   if (This)
   {
     for (unsigned i = 0; i < This->logicalnames.size(); i++)
-      if (This->logicalnames[i] == strip(name) || This->logicalnames[i] == "/dev/" + strip(name))
-        return;	// nothing to add, this logical name already exists
-    if (exists("/dev/" + strip(name)))
+      if (This->logicalnames[i] == n || This->logicalnames[i] == "/dev/" + n)
+        return;                                   // nothing to add, this logical name already exists
+    if ((name[0] != '/') && exists("/dev/" + n))
     {
-      This->logicalnames.push_back("/dev/" + strip(name));
+      This->logicalnames.push_back("/dev/" + n);
     }
     else
-      This->logicalnames.push_back(strip(name));
+      This->logicalnames.push_back(n);
 
-    This->dev = get_devid(strip(name));
+    if(This->dev == "")
+      This->dev = get_devid(n);
   }
 }
 
+
 string hwNode::getDev() const
 {
   if (This)
@@ -908,6 +1011,7 @@ string hwNode::getDev() const
     return "";
 }
 
+
 void hwNode::setDev(const string & s)
 {
   if (This)
@@ -926,6 +1030,7 @@ void hwNode::setDev(const string & s)
   }
 }
 
+
 string hwNode::getBusInfo() const
 {
   if (This)
@@ -934,6 +1039,7 @@ string hwNode::getBusInfo() const
     return "";
 }
 
+
 void hwNode::setBusInfo(const string & businfo)
 {
   if (This)
@@ -941,15 +1047,11 @@ void hwNode::setBusInfo(const string & businfo)
     if (businfo.find('@') != string::npos)
       This->businfo = strip(businfo);
     else
-    {
-      string info = strip(businfo);
-
-      if(matches(info, "^[[:xdigit:]][[:xdigit:]]:[[:xdigit:]][[:xdigit:]]\\.[[:xdigit:]]$"))
-	This->businfo = string("pci@") + info;
-    }
+      This->businfo = guessBusInfo(strip(businfo));
   }
 }
 
+
 string hwNode::getPhysId() const
 {
   if (This)
@@ -958,6 +1060,7 @@ string hwNode::getPhysId() const
     return "";
 }
 
+
 void hwNode::setPhysId(long physid)
 {
   if (This)
@@ -969,8 +1072,9 @@ void hwNode::setPhysId(long physid)
   }
 }
 
+
 void hwNode::setPhysId(unsigned physid1,
-		       unsigned physid2)
+unsigned physid2)
 {
   if (This)
   {
@@ -984,9 +1088,10 @@ void hwNode::setPhysId(unsigned physid1,
   }
 }
 
+
 void hwNode::setPhysId(unsigned physid1,
-		       unsigned physid2,
-		       unsigned physid3)
+unsigned physid2,
+unsigned physid3)
 {
   if (This)
   {
@@ -997,6 +1102,7 @@ void hwNode::setPhysId(unsigned physid1,
   }
 }
 
+
 void hwNode::setPhysId(const string & physid)
 {
   if (This)
@@ -1008,6 +1114,7 @@ void hwNode::setPhysId(const string & physid)
   }
 }
 
+
 void hwNode::assignPhysIds()
 {
   if (!This)
@@ -1023,7 +1130,7 @@ void hwNode::assignPhysIds()
     if (This->children[i].getPhysId() == "")
     {
       while (getChildByPhysId(curid))
-	curid++;
+        curid++;
 
       This->children[i].setPhysId(curid);
     }
@@ -1032,6 +1139,7 @@ void hwNode::assignPhysIds()
   }
 }
 
+
 void hwNode::fixInconsistencies()
 {
   if (!This)
@@ -1040,10 +1148,17 @@ void hwNode::fixInconsistencies()
   if (getCapacity() < getSize())
     setCapacity(0);
 
+  if(getVendor() == "")
+    guessVendor(*this);
+
+  if(getLogicalName() != "")
+    claim();
+
   for (unsigned int i = 0; i < This->children.size(); i++)
     This->children[i].fixInconsistencies();
 }
 
+
 void hwNode::merge(const hwNode & node)
 {
   if (!This)
@@ -1091,18 +1206,19 @@ void hwNode::merge(const hwNode & node)
   for (unsigned int i = 0; i < node.This->features.size(); i++)
     addCapability(node.This->features[i]);
   for (map < string, string >::iterator i = node.This->features_descriptions.begin();
-       i != node.This->features_descriptions.end(); i++)
-    describeCapability(i->first, i->second);
+    i != node.This->features_descriptions.end(); i++)
+  describeCapability(i->first, i->second);
 
   for (map < string, string >::iterator i = node.This->config.begin();
-       i != node.This->config.end(); i++)
-    setConfig(i->first, i->second);
+    i != node.This->config.end(); i++)
+  setConfig(i->first, i->second);
 
   for (map < string, value >::iterator i = node.This->hints.begin();
-       i != node.This->hints.end(); i++)
-    addHint(i->first, i->second);
+    i != node.This->hints.end(); i++)
+  addHint(i->first, i->second);
 }
 
+
 void hwNode::addResource(const resource & r)
 {
   if (!This)
@@ -1111,6 +1227,7 @@ void hwNode::addResource(const resource & r)
   This->resources.push_back(r);
 }
 
+
 bool hwNode::usesResource(const resource & r) const
 {
   if (!This)
@@ -1123,6 +1240,7 @@ bool hwNode::usesResource(const resource & r) const
   return false;
 }
 
+
 vector < string > hwNode::getResources(const string & separator) const
 {
   vector < string > result;
@@ -1131,18 +1249,20 @@ vector < string > hwNode::getResources(const string & separator) const
     return result;
 
   for (vector < resource >::iterator i = This->resources.begin();
-       i != This->resources.end(); i++)
-    result.push_back(i->asString(separator));
+    i != This->resources.end(); i++)
+  result.push_back(i->asString(separator));
 
   return result;
 }
 
+
 void hwNode::setWidth(unsigned int width)
 {
   if(This)
     This->width = width;
 }
 
+
 unsigned int hwNode::getWidth() const
 {
   if(This)
@@ -1151,12 +1271,14 @@ unsigned int hwNode::getWidth() const
     return 0;
 }
 
+
 void hwNode::addHint(const string & id, const value & v)
 {
   if(This)
     This->hints[id] = v;
 }
 
+
 value hwNode::getHint(const string & id) const
 {
   if(!This)
@@ -1165,6 +1287,7 @@ value hwNode::getHint(const string & id) const
   return This->hints[id];
 }
 
+
 string hwNode::asXML(unsigned level)
 {
   vector < string > config;
@@ -1173,17 +1296,28 @@ string hwNode::asXML(unsigned level)
 
   if(!This) return "";
 
-  config = getConfigValues("\" value=\"");
+  config = getConfigKeys();
   resources = getResources("\" value=\"");
 
   if (level == 0)
   {
+    struct utsname un;
+
     out << "<?xml version=\"1.0\" standalone=\"yes\" ?>" << endl;
     out << "<!-- generated by lshw-" << getpackageversion() << " -->" <<
-#if defined(__GNUC__) && defined(__VERSION__)
-    endl << "<!-- GCC " << __VERSION__ << " -->" <<
-#endif
+  #if defined(__GNUC__) && defined(__VERSION__)
+      endl << "<!-- GCC " << __VERSION__ << " -->" <<
+  #endif
       endl;
+
+    if(uname(&un) == 0)
+      out << "<!-- " << un.sysname << " " << un.release << " " << un.version << " " << un.machine << " -->" << endl;
+  #if defined(__GLIBC__)
+    char version[PATH_MAX];
+
+      if(confstr(_CS_GNU_LIBC_VERSION, version, sizeof(version))>0)
+        out << "<!-- GNU libc " << __GLIBC__ << " (" << version << ") -->" << endl;
+  #endif
     if (geteuid() != 0)
       out << "<!-- WARNING: not running as root -->" << endl;
   }
@@ -1252,11 +1386,11 @@ string hwNode::asXML(unsigned level)
 
       for(unsigned int i = 0; i<logicalnames.size(); i++)
       {
-      	out << spaces(2*level+1);
+        out << spaces(2*level+1);
         out << "<logicalname>";
-      	out << logicalnames[i];
+        out << logicalnames[i];
         out << "</logicalname>";
-      	out << endl;
+        out << endl;
       }
     }
 
@@ -1282,11 +1416,11 @@ string hwNode::asXML(unsigned level)
     {
       out << spaces(2*level+1);
       out << "<serial>";
-      out << escape(getSerial());
+      out << (::enabled("output:sanitize")?REMOVED:escape(getSerial()));
       out << "</serial>";
       out << endl;
     }
-  
+
     if (getSlot() != "")
     {
       out << spaces(2*level+1);
@@ -1295,65 +1429,65 @@ string hwNode::asXML(unsigned level)
       out << "</slot>";
       out << endl;
     }
-  
+
     if (getSize() > 0)
     {
       out << spaces(2*level+1);
       out << "<size";
       switch (getClass())
       {
-      case hw::memory:
-      case hw::address:
-      case hw::storage:
-      case hw::disk:
-      case hw::display:
-        out << " units=\"bytes\"";
-        break;
-  
-      case hw::processor:
-      case hw::bus:
-      case hw::system:
-        out << " units=\"Hz\"";
-        break;
-  
-      case hw::power:
-        out << " units=\"mWh\"";
-        break;
-  
-      default:
-        out << "";
+        case hw::memory:
+        case hw::address:
+        case hw::storage:
+        case hw::disk:
+        case hw::display:
+          out << " units=\"bytes\"";
+          break;
+
+        case hw::processor:
+        case hw::bus:
+        case hw::system:
+          out << " units=\"Hz\"";
+          break;
+
+        case hw::power:
+          out << " units=\"mWh\"";
+          break;
+
+        default:
+          out << "";
       }
       out << ">";
       out << getSize();
       out << "</size>";
       out << endl;
     }
-  
+
     if (getCapacity() > 0)
     {
       out << spaces(2*level+1);
       out << "<capacity";
       switch (getClass())
       {
-      case hw::memory:
-      case hw::address:
-      case hw::storage:
-      case hw::disk:
-        out << " units=\"bytes\"";
-        break;
-  
-      case hw::processor:
-      case hw::bus:
-      case hw::system:
-        out << " units=\"Hz\"";
-        break;
-  
-      case hw::power:
-        out << " units=\"mWh\"";
-        break;
-  
-      default:
-        out << "";
+        case hw::memory:
+        case hw::address:
+        case hw::storage:
+        case hw::disk:
+          out << " units=\"bytes\"";
+          break;
+
+        case hw::processor:
+        case hw::bus:
+        case hw::system:
+          out << " units=\"Hz\"";
+          break;
+
+        case hw::power:
+          out << " units=\"mWh\"";
+          break;
+
+        default:
+          out << "";
       }
       out << ">";
       out << getCapacity();
@@ -1386,14 +1520,14 @@ string hwNode::asXML(unsigned level)
       for (unsigned int j = 0; j < config.size(); j++)
       {
         out << spaces(2*level+2);
-        out << "<setting id=\"" << escape(config[j]) << "\" />";
+        out << "<setting id=\"" << escape(config[j]) << "\" value=\"" << escape(getConfig(config[j])) << "\" />";
         out << endl;
       }
       out << spaces(2*level+1);
       out << "</configuration>" << endl;
     }
     config.clear();
-  
+
     splitlines(getCapabilities(), config, ' ');
     if (config.size() > 0)
     {
@@ -1405,13 +1539,13 @@ string hwNode::asXML(unsigned level)
         out << "<capability id=\"" << escape(config[j]);
         if (getCapabilityDescription(config[j]) == "")
         {
-	  out << "\" />";
+          out << "\" />";
         }
         else
         {
-	  out << "\" >";
-	  out << escape(getCapabilityDescription(config[j]));
-	  out << "</capability>";
+          out << "\" >";
+          out << escape(getCapabilityDescription(config[j]));
+          out << "</capability>";
         }
         out << endl;
       }
@@ -1447,10 +1581,45 @@ string hwNode::asXML(unsigned level)
     out << "</node>" << endl;
   }
 
-
   return out.str();
 }
 
+string hwNode::asString()
+{
+  string summary = "";
+  if(!This)
+    return "";
+
+  if(getClass() != hw::memory)
+    summary = getProduct();        // memory devices tend to have obscure product names
+  if (summary == "")
+    summary = getDescription();
+
+  if((getClass() == hw::memory) || (getClass() == hw::disk) || (getClass() == hw::storage) || (getClass() == hw::volume))
+  {
+    unsigned long long size = 0;
+    if(getClass() != hw::memory)
+    {
+      if(getCapacity())
+        size = getCapacity();
+      else
+      {
+        if(getSize())
+          size = getSize();
+      }
+    }
+    else
+    {
+      if(getSize())
+        size = getSize();
+    }
+    if(size)
+      summary = (getClass()==hw::disk?(decimalkilos(size)+"B"):kilobytes(size)) +" "+ summary;
+  }
+
+  return summary;
+}
+
 struct hw::resource_i
 {
   hw::hwResourceType type;
@@ -1474,6 +1643,7 @@ resource::resource()
   }
 }
 
+
 resource::~resource()
 {
   if (This)
@@ -1488,6 +1658,7 @@ resource::~resource()
   }
 }
 
+
 resource::resource(const resource & r)
 {
   This = r.This;
@@ -1496,13 +1667,14 @@ resource::resource(const resource & r)
     This->refcount++;
 }
 
+
 resource & resource::operator = (const resource & r)
 {
   if (this == &r)
-    return *this;		// ignore self-affectation
+    return *this;                                 // ignore self-affectation
 
   if (This == r.This)
-    return *this;		// both objects reference the same data
+    return *this;                                 // both objects reference the same data
 
   if (This)
   {
@@ -1522,8 +1694,9 @@ resource & resource::operator = (const resource & r)
   return *this;
 }
 
+
 resource resource::iomem(unsigned long long start,
-			 unsigned long long end)
+unsigned long long end)
 {
   resource r;
 
@@ -1537,8 +1710,9 @@ resource resource::iomem(unsigned long long start,
   return r;
 }
 
+
 resource resource::ioport(unsigned long start,
-			  unsigned long end)
+unsigned long end)
 {
   resource r;
 
@@ -1552,8 +1726,9 @@ resource resource::ioport(unsigned long start,
   return r;
 }
 
+
 resource resource::mem(unsigned long long start,
-		       unsigned long long end)
+unsigned long long end)
 {
   resource r;
 
@@ -1567,6 +1742,7 @@ resource resource::mem(unsigned long long start,
   return r;
 }
 
+
 resource resource::irq(unsigned int value)
 {
   resource r;
@@ -1580,6 +1756,7 @@ resource resource::irq(unsigned int value)
   return r;
 }
 
+
 resource resource::dma(unsigned int value)
 {
   resource r;
@@ -1593,6 +1770,7 @@ resource resource::dma(unsigned int value)
   return r;
 }
 
+
 string resource::asString(const string & separator) const
 {
   char buffer[80];
@@ -1605,75 +1783,78 @@ string resource::asString(const string & separator) const
 
   switch (This->type)
   {
-  case hw::none:
-    result = "(none)";
-    break;
-  case hw::dma:
-    result = "dma" + separator;
-    snprintf(buffer, sizeof(buffer), "%d", This->ui1);
-    break;
-  case hw::irq:
-    result = "irq" + separator;
-    snprintf(buffer, sizeof(buffer), "%d", This->ui1);
-    break;
-  case hw::iomem:
-    result = "iomemory" + separator;
-    snprintf(buffer, sizeof(buffer), "%llx-%llx", This->ull1, This->ull2);
-    break;
-  case hw::mem:
-    result = "memory" + separator;
-    snprintf(buffer, sizeof(buffer), "%llx-%llx", This->ull1, This->ull2);
-    break;
-  case hw::ioport:
-    result = "ioport" + separator;
-    snprintf(buffer, sizeof(buffer), "%lx-%lx", This->ul1, This->ul2);
-    break;
-  default:
-    result = "(unknown)";
+    case hw::none:
+      result = "(none)";
+      break;
+    case hw::dma:
+      result = "dma" + separator;
+      snprintf(buffer, sizeof(buffer), "%d", This->ui1);
+      break;
+    case hw::irq:
+      result = "irq" + separator;
+      snprintf(buffer, sizeof(buffer), "%d", This->ui1);
+      break;
+    case hw::iomem:
+      result = "iomemory" + separator;
+      snprintf(buffer, sizeof(buffer), "%llx-%llx", This->ull1, This->ull2);
+      break;
+    case hw::mem:
+      result = "memory" + separator;
+      snprintf(buffer, sizeof(buffer), "%llx-%llx", This->ull1, This->ull2);
+      break;
+    case hw::ioport:
+      result = "ioport" + separator;
+      snprintf(buffer, sizeof(buffer), "%lx-%lx", This->ul1, This->ul2);
+      break;
+    default:
+      result = "(unknown)";
   }
 
   return result + string(buffer);
 }
 
+
 bool resource::operator == (const resource & r)
-     const {
-       if (This == r.This)
-	 return true;
+const
+{
+  if (This == r.This)
+    return true;
 
-       if (!This || !r.This)
-	 return false;
+  if (!This || !r.This)
+    return false;
 
-       if (This->type != r.This->type)
-	 return false;
+  if (This->type != r.This->type)
+    return false;
 
-       switch (This->type)
-       {
-       case hw::dma:
-       case hw::irq:
-	 return This->ui1 == r.This->ui1;
-	 break;
+  switch (This->type)
+  {
+    case hw::dma:
+    case hw::irq:
+      return This->ui1 == r.This->ui1;
+      break;
 
-	 case hw::iomem:case hw::mem:return (This->ull1 >= r.This->ull1)
-	   && (This->ull2 <= r.This->ull2) || (r.This->ull1 >= This->ull1)
-	   && (r.This->ull2 <= This->ull2);
-	 break;
+    case hw::iomem:case hw::mem:return (This->ull1 >= r.This->ull1)
+      && (This->ull2 <= r.This->ull2) || (r.This->ull1 >= This->ull1)
+      && (r.This->ull2 <= This->ull2);
+    break;
 
-	 case hw::ioport:return (This->ul1 >= r.This->ul1)
-	   && (This->ul1 <= r.This->ul2) || (r.This->ul1 >= This->ul1)
-	   && (r.This->ul1 <= This->ul2);
-	 break;
+    case hw::ioport:return (This->ul1 >= r.This->ul1)
+      && (This->ul1 <= r.This->ul2) || (r.This->ul1 >= This->ul1)
+      && (r.This->ul1 <= This->ul2);
+    break;
+
+    default:return false;
+  }
+}
 
-	 default:return false;
-       }
-     }
 
 struct hw::value_i
 {
   hw::hwValueType type;
 
-    long long ll;
-    string s;
-    bool b;
+  long long ll;
+  string s;
+  bool b;
 
   int refcount;
 };
@@ -1689,6 +1870,7 @@ value::value()
   }
 }
 
+
 value::~value()
 {
   if(This)
@@ -1700,6 +1882,7 @@ value::~value()
   }
 }
 
+
 value::value(long long ll)
 {
   This = new hw::value_i;
@@ -1712,6 +1895,7 @@ value::value(long long ll)
   }
 }
 
+
 value::value(const string & s)
 {
   This = new hw::value_i;
@@ -1724,6 +1908,7 @@ value::value(const string & s)
   }
 }
 
+
 value::value(const value & v)
 {
   This = v.This;
@@ -1734,6 +1919,7 @@ value::value(const value & v)
   }
 }
 
+
 value & value::operator=(const value & v)
 {
   if(v.This == This) return *this;
@@ -1752,6 +1938,7 @@ value & value::operator=(const value & v)
   return *this;
 }
 
+
 bool value::operator==(const value & v) const
 {
   if(v.This == This) return true;
@@ -1795,6 +1982,7 @@ string value::asString() const
   return "";
 }
 
+
 bool value::defined() const
 {
   if(!This) return false;
@@ -1802,4 +1990,3 @@ bool value::defined() const
 
   return true;
 }
-
