@@ -8,10 +8,11 @@
  *  kernel itself.
  */
 
+#include "version.h"
 #include "isapnp.h"
 #include "pnp.h"
 
-static const char rcsid[] = "@(#) $Id: isapnp.cc 1319 2006-05-09 07:48:12Z ezix $";
+__ID("@(#) $Id: isapnp.cc 1907 2007-10-13 22:23:16Z lyonel $");
 
 #ifdef __i386__
 
@@ -23,99 +24,99 @@ static const char rcsid[] = "@(#) $Id: isapnp.cc 1319 2006-05-09 07:48:12Z ezix
 #include <string.h>
 
 /* config parameters */
-#define PNP_CONFIG_NORMAL	0x0001
-#define PNP_CONFIG_FORCE	0x0002	/* disables validity checking */
+#define PNP_CONFIG_NORMAL 0x0001
+#define PNP_CONFIG_FORCE  0x0002                  /* disables validity checking */
 
 /* capabilities */
-#define PNP_READ		0x0001
-#define PNP_WRITE		0x0002
-#define PNP_DISABLE		0x0004
-#define PNP_CONFIGURABLE	0x0008
-#define PNP_REMOVABLE		0x0010
+#define PNP_READ    0x0001
+#define PNP_WRITE   0x0002
+#define PNP_DISABLE   0x0004
+#define PNP_CONFIGURABLE  0x0008
+#define PNP_REMOVABLE   0x0010
 
 /* status */
-#define PNP_READY		0x0000
-#define PNP_ATTACHED		0x0001
-#define PNP_BUSY		0x0002
-#define PNP_FAULTY		0x0004
+#define PNP_READY   0x0000
+#define PNP_ATTACHED    0x0001
+#define PNP_BUSY    0x0002
+#define PNP_FAULTY    0x0004
 
-#define ISAPNP_CFG_ACTIVATE		0x30	/* byte */
-#define ISAPNP_CFG_MEM			0x40	/* 4 * dword */
-#define ISAPNP_CFG_PORT			0x60	/* 8 * word */
-#define ISAPNP_CFG_IRQ			0x70	/* 2 * word */
-#define ISAPNP_CFG_DMA			0x74	/* 2 * byte */
+#define ISAPNP_CFG_ACTIVATE   0x30                /* byte */
+#define ISAPNP_CFG_MEM      0x40                  /* 4 * dword */
+#define ISAPNP_CFG_PORT     0x60                  /* 8 * word */
+#define ISAPNP_CFG_IRQ      0x70                  /* 2 * word */
+#define ISAPNP_CFG_DMA      0x74                  /* 2 * byte */
 
-#define ISAPNP_VENDOR(a,b,c)	(((((a)-'A'+1)&0x3f)<<2)|\
-				((((b)-'A'+1)&0x18)>>3)|((((b)-'A'+1)&7)<<13)|\
-				((((c)-'A'+1)&0x1f)<<8))
-#define ISAPNP_DEVICE(x)	((((x)&0xf000)>>8)|\
-				 (((x)&0x0f00)>>8)|\
-				 (((x)&0x00f0)<<8)|\
-				 (((x)&0x000f)<<8))
-#define ISAPNP_FUNCTION(x)	ISAPNP_DEVICE(x)
+#define ISAPNP_VENDOR(a,b,c)  (((((a)-'A'+1)&0x3f)<<2)|\
+((((b)-'A'+1)&0x18)>>3)|((((b)-'A'+1)&7)<<13)|\
+((((c)-'A'+1)&0x1f)<<8))
+#define ISAPNP_DEVICE(x)  ((((x)&0xf000)>>8)|\
+(((x)&0x0f00)>>8)|\
+(((x)&0x00f0)<<8)|\
+(((x)&0x000f)<<8))
+#define ISAPNP_FUNCTION(x)  ISAPNP_DEVICE(x)
 
 #define DEVICE_COUNT_COMPATIBLE 4
 
-#define ISAPNP_ANY_ID		0xffff
-#define ISAPNP_CARD_DEVS	8
+#define ISAPNP_ANY_ID   0xffff
+#define ISAPNP_CARD_DEVS  8
 
 #define ISAPNP_CARD_ID(_va, _vb, _vc, _device) \
-		.card_vendor = ISAPNP_VENDOR(_va, _vb, _vc), .card_device = ISAPNP_DEVICE(_device)
+.card_vendor = ISAPNP_VENDOR(_va, _vb, _vc), .card_device = ISAPNP_DEVICE(_device)
 #define ISAPNP_CARD_END \
-		.card_vendor = 0, .card_device = 0
+.card_vendor = 0, .card_device = 0
 #define ISAPNP_DEVICE_ID(_va, _vb, _vc, _function) \
-		{ .vendor = ISAPNP_VENDOR(_va, _vb, _vc), .function = ISAPNP_FUNCTION(_function) }
+{ .vendor = ISAPNP_VENDOR(_va, _vb, _vc), .function = ISAPNP_FUNCTION(_function) }
 
 struct isapnp_card_id
 {
-  unsigned long driver_data;	/* data private to the driver */
+  unsigned long driver_data;                      /* data private to the driver */
   unsigned short card_vendor, card_device;
   struct
   {
     unsigned short vendor, function;
   }
-  devs[ISAPNP_CARD_DEVS];	/* logical devices */
+  devs[ISAPNP_CARD_DEVS];                         /* logical devices */
 };
 
 #define ISAPNP_DEVICE_SINGLE(_cva, _cvb, _cvc, _cdevice, _dva, _dvb, _dvc, _dfunction) \
-		.card_vendor = ISAPNP_VENDOR(_cva, _cvb, _cvc), .card_device =  ISAPNP_DEVICE(_cdevice), \
-		.vendor = ISAPNP_VENDOR(_dva, _dvb, _dvc), .function = ISAPNP_FUNCTION(_dfunction)
+.card_vendor = ISAPNP_VENDOR(_cva, _cvb, _cvc), .card_device =  ISAPNP_DEVICE(_cdevice), \
+.vendor = ISAPNP_VENDOR(_dva, _dvb, _dvc), .function = ISAPNP_FUNCTION(_dfunction)
 #define ISAPNP_DEVICE_SINGLE_END \
-		.card_vendor = 0, .card_device = 0
+.card_vendor = 0, .card_device = 0
 
 struct isapnp_device_id
 {
   unsigned short card_vendor, card_device;
   unsigned short vendor, function;
-  unsigned long driver_data;	/* data private to the driver */
+  unsigned long driver_data;                      /* data private to the driver */
 };
 
-static int isapnp_rdp;		/* Read Data Port */
-static int isapnp_reset = 0;	/* reset all PnP cards (deactivate) */
+static int isapnp_rdp;                            /* Read Data Port */
+static int isapnp_reset = 0;                      /* reset all PnP cards (deactivate) */
 static bool isapnp_error = false;
 
-#define _PIDXR		0x279
-#define _PNPWRP		0xa79
+#define _PIDXR    0x279
+#define _PNPWRP   0xa79
 
 /* short tags */
-#define _STAG_PNPVERNO		0x01
-#define _STAG_LOGDEVID		0x02
-#define _STAG_COMPATDEVID	0x03
-#define _STAG_IRQ		0x04
-#define _STAG_DMA		0x05
-#define _STAG_STARTDEP		0x06
-#define _STAG_ENDDEP		0x07
-#define _STAG_IOPORT		0x08
-#define _STAG_FIXEDIO		0x09
-#define _STAG_VENDOR		0x0e
-#define _STAG_END		0x0f
+#define _STAG_PNPVERNO    0x01
+#define _STAG_LOGDEVID    0x02
+#define _STAG_COMPATDEVID 0x03
+#define _STAG_IRQ   0x04
+#define _STAG_DMA   0x05
+#define _STAG_STARTDEP    0x06
+#define _STAG_ENDDEP    0x07
+#define _STAG_IOPORT    0x08
+#define _STAG_FIXEDIO   0x09
+#define _STAG_VENDOR    0x0e
+#define _STAG_END   0x0f
 /* long tags */
-#define _LTAG_MEMRANGE		0x81
-#define _LTAG_ANSISTR		0x82
-#define _LTAG_UNICODESTR	0x83
-#define _LTAG_VENDOR		0x84
-#define _LTAG_MEM32RANGE	0x85
-#define _LTAG_FIXEDMEM32RANGE	0x86
+#define _LTAG_MEMRANGE    0x81
+#define _LTAG_ANSISTR   0x82
+#define _LTAG_UNICODESTR  0x83
+#define _LTAG_VENDOR    0x84
+#define _LTAG_MEM32RANGE  0x85
+#define _LTAG_FIXEDMEM32RANGE 0x86
 
 static unsigned char isapnp_checksum_value;
 static int isapnp_detected;
@@ -132,6 +133,7 @@ static void udelay(unsigned long l)
   nanosleep(&delay, NULL);
 }
 
+
 static bool write_data(unsigned char x)
 {
   int fd = open("/dev/port", O_WRONLY);
@@ -151,6 +153,7 @@ static bool write_data(unsigned char x)
   }
 }
 
+
 static bool write_address(unsigned char x)
 {
   int fd = open("/dev/port", O_WRONLY);
@@ -170,6 +173,7 @@ static bool write_address(unsigned char x)
   }
 }
 
+
 static unsigned char read_data(void)
 {
   int fd = open("/dev/port", O_RDONLY);
@@ -188,6 +192,7 @@ static unsigned char read_data(void)
   return val;
 }
 
+
 static unsigned char isapnp_read_byte(unsigned char idx)
 {
   if (write_address(idx))
@@ -196,12 +201,14 @@ static unsigned char isapnp_read_byte(unsigned char idx)
     return 0;
 }
 
+
 static bool isapnp_write_byte(unsigned char idx,
-			      unsigned char val)
+unsigned char val)
 {
   return write_address(idx) && write_data(val);
 }
 
+
 static bool isapnp_key(void)
 {
   unsigned char code = 0x6a, msb;
@@ -227,19 +234,22 @@ static bool isapnp_key(void)
   return true;
 }
 
+
 /* place all pnp cards in wait-for-key state */
 static bool isapnp_wait(void)
 {
   return isapnp_write_byte(0x02, 0x02);
 }
 
+
 static bool isapnp_wake(unsigned char csn)
 {
   return isapnp_write_byte(0x03, csn);
 }
 
+
 static bool isapnp_peek(unsigned char *data,
-			int bytes)
+int bytes)
 {
   int i, j;
   unsigned char d = 0;
@@ -252,18 +262,18 @@ static bool isapnp_peek(unsigned char *data,
     {
       d = isapnp_read_byte(0x05);
       if (isapnp_error)
-	return false;
+        return false;
       if (d & 1)
-	break;
+        break;
       udelay(10);
     }
     if (!(d & 1))
     {
       if (data != NULL)
-	*data++ = 0xff;
+        *data++ = 0xff;
       continue;
     }
-    d = isapnp_read_byte(0x04);	/* PRESDI */
+    d = isapnp_read_byte(0x04);                   /* PRESDI */
     isapnp_checksum_value += d;
     if (data != NULL)
       *data++ = d;
@@ -271,17 +281,18 @@ static bool isapnp_peek(unsigned char *data,
   return true;
 }
 
-#define RDP_STEP	32	/* minimum is 4 */
+
+#define RDP_STEP  32                              /* minimum is 4 */
 
 static int isapnp_next_rdp(void)
 {
   int rdp = isapnp_rdp;
   while (rdp <= 0x3ff)
   {
-    /*
-     *      We cannot use NE2000 probe spaces for ISAPnP or we
-     *      will lock up machines.
-     */
+/*
+ *      We cannot use NE2000 probe spaces for ISAPnP or we
+ *      will lock up machines.
+ */
     if ((rdp < 0x280 || rdp > 0x380) /*&& !check_region(rdp, 1) */ )
     {
       isapnp_rdp = rdp;
@@ -292,6 +303,7 @@ static int isapnp_next_rdp(void)
   return -1;
 }
 
+
 /* Set read port address */
 static inline bool isapnp_set_rdp(void)
 {
@@ -304,6 +316,7 @@ static inline bool isapnp_set_rdp(void)
     return false;
 }
 
+
 /*
  *	Perform an isolation. The port selection code now tries to avoid
  *	"dangerous to read" ports.
@@ -316,9 +329,9 @@ static int isapnp_isolate_rdp_select(void)
   if (!isapnp_key())
     return -1;
 
-  /*
-   * Control: reset CSN and conditionally everything else too 
-   */
+/*
+ * Control: reset CSN and conditionally everything else too
+ */
   if (!isapnp_write_byte(0x02, isapnp_reset ? 0x05 : 0x04))
     return -1;
   udelay(2);
@@ -344,6 +357,7 @@ static int isapnp_isolate_rdp_select(void)
   return 0;
 }
 
+
 /*
  *  Isolate (assign uniqued CSN) to all ISA PnP devices.
  */
@@ -369,48 +383,48 @@ static int isapnp_isolate(void)
     for (i = 1; i <= 64; i++)
     {
       if (isapnp_error)
-	return -1;
+        return -1;
 
       data = read_data() << 8;
       udelay(250);
       data = data | read_data();
       udelay(250);
       if (data == 0x55aa)
-	bit = 0x01;
+        bit = 0x01;
       checksum =
-	((((checksum ^ (checksum >> 1)) & 0x01) ^ bit) << 7) | (checksum >>
-								1);
+        ((((checksum ^ (checksum >> 1)) & 0x01) ^ bit) << 7) | (checksum >>
+        1);
       bit = 0x00;
     }
     for (i = 65; i <= 72; i++)
     {
       if (isapnp_error)
-	return -1;
+        return -1;
 
       data = read_data() << 8;
       udelay(250);
       data = data | read_data();
       udelay(250);
       if (data == 0x55aa)
-	chksum |= (1 << (i - 65));
+        chksum |= (1 << (i - 65));
     }
     if (checksum != 0x00 && checksum == chksum)
     {
       csn++;
 
       if (isapnp_error)
-	return -1;
+        return -1;
 
       if (!isapnp_write_byte(0x06, csn))
-	return -1;
+        return -1;
       udelay(250);
       iteration++;
       if (!isapnp_wake(0x00))
-	return -1;
+        return -1;
       isapnp_set_rdp();
       udelay(1000);
       if (!write_address(0x01))
-	return -1;
+        return -1;
       udelay(1000);
       goto __next;
     }
@@ -418,13 +432,13 @@ static int isapnp_isolate(void)
     {
       isapnp_rdp += RDP_STEP;
       if (isapnp_isolate_rdp_select() < 0)
-	return -1;
+        return -1;
     }
     else if (iteration > 1)
     {
       break;
     }
-  __next:
+    __next:
     checksum = 0x6a;
     chksum = 0x00;
     bit = 0x00;
@@ -434,20 +448,21 @@ static int isapnp_isolate(void)
   return csn;
 }
 
+
 /*
  *  Read one tag from stream.
  */
 
 static int isapnp_read_tag(unsigned char *type,
-			   unsigned short *size)
+unsigned short *size)
 {
   unsigned char tag, tmp[2];
 
   isapnp_peek(&tag, 1);
-  if (tag == 0)			/* invalid tag */
+  if (tag == 0)                                   /* invalid tag */
     return -1;
   if (tag & 0x80)
-  {				/* large item */
+  {                                               /* large item */
     *type = tag;
     isapnp_peek(tmp, 2);
     *size = (tmp[1] << 8) | tmp[0];
@@ -459,15 +474,16 @@ static int isapnp_read_tag(unsigned char *type,
   }
 #if 0
   printk(KERN_DEBUG "tag = 0x%x, type = 0x%x, size = %i\n", tag, *type,
-	 *size);
+    *size);
 #endif
-  if (type == 0)		/* wrong type */
+  if (type == 0)                                  /* wrong type */
     return -1;
-  if (*type == 0xff && *size == 0xffff)	/* probably invalid data */
+  if (*type == 0xff && *size == 0xffff)           /* probably invalid data */
     return -1;
   return 0;
 }
 
+
 /*
  *  Skip specified number of bytes from stream.
  */
@@ -477,64 +493,68 @@ static void isapnp_skip_bytes(int count)
   isapnp_peek(NULL, count);
 }
 
+
 /*
  *  Parse EISA id.
  */
 
 static const char *isapnp_parse_id(unsigned short vendor,
-				   unsigned short device)
+unsigned short device)
 {
   static char id[8];
   snprintf(id, sizeof(id), "%c%c%c%x%x%x%x",
-	   'A' + ((vendor >> 2) & 0x3f) - 1,
-	   'A' + (((vendor & 3) << 3) | ((vendor >> 13) & 7)) - 1,
-	   'A' + ((vendor >> 8) & 0x1f) - 1,
-	   (device >> 4) & 0x0f,
-	   device & 0x0f, (device >> 12) & 0x0f, (device >> 8) & 0x0f);
+    'A' + ((vendor >> 2) & 0x3f) - 1,
+    'A' + (((vendor & 3) << 3) | ((vendor >> 13) & 7)) - 1,
+    'A' + ((vendor >> 8) & 0x1f) - 1,
+    (device >> 4) & 0x0f,
+    device & 0x0f, (device >> 12) & 0x0f, (device >> 8) & 0x0f);
 
   return id;
 }
 
+
 /*
  *  Parse logical device tag.
  */
 
 static hwNode *isapnp_parse_device(hwNode & card,
-				   int size,
-				   int number)
+int size,
+int number)
 {
   unsigned char tmp[6];
 
   isapnp_peek(tmp, size);
   return card.
     addChild(hwNode
-	     (isapnp_parse_id
-	      ((tmp[1] << 8) | tmp[0], (tmp[3] << 8) | tmp[2])));
+    (isapnp_parse_id
+    ((tmp[1] << 8) | tmp[0], (tmp[3] << 8) | tmp[2])));
 }
 
+
 /*
  *  Add IRQ resource to resources list.
  */
 
 static void isapnp_parse_irq_resource(hwNode & n,
-				      int size)
+int size)
 {
   unsigned char tmp[3];
 
   isapnp_peek(tmp, size);
-  /*
-   * irq = isapnp_alloc(sizeof(struct pnp_irq));
-   * if (!irq)
-   * return;
-   * irq->map = (tmp[1] << 8) | tmp[0];
-   * if (size > 2)
-   * irq->flags = tmp[2];
-   * else
-   * irq->flags = IORESOURCE_IRQ_HIGHEDGE;
-   * return;
-   */
+/*
+ * irq = isapnp_alloc(sizeof(struct pnp_irq));
+ * if (!irq)
+ * return;
+ * irq->map = (tmp[1] << 8) | tmp[0];
+ * if (size > 2)
+ * irq->flags = tmp[2];
+ * else
+ * irq->flags = IORESOURCE_IRQ_HIGHEDGE;
+ * return;
+ */
 }
 
+
 /*
  *  Add DMA resource to resources list.
  */
@@ -544,39 +564,41 @@ static void isapnp_parse_dma_resource(int size)
   unsigned char tmp[2];
 
   isapnp_peek(tmp, size);
-  /*
-   * dma = isapnp_alloc(sizeof(struct pnp_dma));
-   * if (!dma)
-   * return;
-   * dma->map = tmp[0];
-   * dma->flags = tmp[1];
-   * return;
-   */
+/*
+ * dma = isapnp_alloc(sizeof(struct pnp_dma));
+ * if (!dma)
+ * return;
+ * dma->map = tmp[0];
+ * dma->flags = tmp[1];
+ * return;
+ */
 }
 
+
 /*
  *  Add port resource to resources list.
  */
 
 static void isapnp_parse_port_resource(hwNode & n,
-				       int size)
+int size)
 {
   unsigned char tmp[7];
 
   isapnp_peek(tmp, size);
-  /*
-   * port = isapnp_alloc(sizeof(struct pnp_port));
-   * if (!port)
-   * return;
-   * port->min = (tmp[2] << 8) | tmp[1];
-   * port->max = (tmp[4] << 8) | tmp[3];
-   * port->align = tmp[5];
-   * port->size = tmp[6];
-   * port->flags = tmp[0] ? PNP_PORT_FLAG_16BITADDR : 0;
-   * return;
-   */
+/*
+ * port = isapnp_alloc(sizeof(struct pnp_port));
+ * if (!port)
+ * return;
+ * port->min = (tmp[2] << 8) | tmp[1];
+ * port->max = (tmp[4] << 8) | tmp[3];
+ * port->align = tmp[5];
+ * port->size = tmp[6];
+ * port->flags = tmp[0] ? PNP_PORT_FLAG_16BITADDR : 0;
+ * return;
+ */
 }
 
+
 /*
  *  Add fixed port resource to resources list.
  */
@@ -586,18 +608,19 @@ static void isapnp_parse_fixed_port_resource(int size)
   unsigned char tmp[3];
 
   isapnp_peek(tmp, size);
-  /*
-   * port = isapnp_alloc(sizeof(struct pnp_port));
-   * if (!port)
-   * return;
-   * port->min = port->max = (tmp[1] << 8) | tmp[0];
-   * port->size = tmp[2];
-   * port->align = 0;
-   * port->flags = PNP_PORT_FLAG_FIXED;
-   * return;
-   */
+/*
+ * port = isapnp_alloc(sizeof(struct pnp_port));
+ * if (!port)
+ * return;
+ * port->min = port->max = (tmp[1] << 8) | tmp[0];
+ * port->size = tmp[2];
+ * port->align = 0;
+ * port->flags = PNP_PORT_FLAG_FIXED;
+ * return;
+ */
 }
 
+
 /*
  *  Add memory resource to resources list.
  */
@@ -607,19 +630,20 @@ static void isapnp_parse_mem_resource(int size)
   unsigned char tmp[9];
 
   isapnp_peek(tmp, size);
-  /*
-   * mem = isapnp_alloc(sizeof(struct pnp_mem));
-   * if (!mem)
-   * return;
-   * mem->min = ((tmp[2] << 8) | tmp[1]) << 8;
-   * mem->max = ((tmp[4] << 8) | tmp[3]) << 8;
-   * mem->align = (tmp[6] << 8) | tmp[5];
-   * mem->size = ((tmp[8] << 8) | tmp[7]) << 8;
-   * mem->flags = tmp[0];
-   * return;
-   */
+/*
+ * mem = isapnp_alloc(sizeof(struct pnp_mem));
+ * if (!mem)
+ * return;
+ * mem->min = ((tmp[2] << 8) | tmp[1]) << 8;
+ * mem->max = ((tmp[4] << 8) | tmp[3]) << 8;
+ * mem->align = (tmp[6] << 8) | tmp[5];
+ * mem->size = ((tmp[8] << 8) | tmp[7]) << 8;
+ * mem->flags = tmp[0];
+ * return;
+ */
 }
 
+
 /*
  *  Add 32-bit memory resource to resources list.
  */
@@ -629,18 +653,19 @@ static void isapnp_parse_mem32_resource(int size)
   unsigned char tmp[17];
 
   isapnp_peek(tmp, size);
-  /*
-   * mem = isapnp_alloc(sizeof(struct pnp_mem));
-   * if (!mem)
-   * return;
-   * mem->min = (tmp[4] << 24) | (tmp[3] << 16) | (tmp[2] << 8) | tmp[1];
-   * mem->max = (tmp[8] << 24) | (tmp[7] << 16) | (tmp[6] << 8) | tmp[5];
-   * mem->align = (tmp[12] << 24) | (tmp[11] << 16) | (tmp[10] << 8) | tmp[9];
-   * mem->size = (tmp[16] << 24) | (tmp[15] << 16) | (tmp[14] << 8) | tmp[13];
-   * mem->flags = tmp[0];
-   */
+/*
+ * mem = isapnp_alloc(sizeof(struct pnp_mem));
+ * if (!mem)
+ * return;
+ * mem->min = (tmp[4] << 24) | (tmp[3] << 16) | (tmp[2] << 8) | tmp[1];
+ * mem->max = (tmp[8] << 24) | (tmp[7] << 16) | (tmp[6] << 8) | tmp[5];
+ * mem->align = (tmp[12] << 24) | (tmp[11] << 16) | (tmp[10] << 8) | tmp[9];
+ * mem->size = (tmp[16] << 24) | (tmp[15] << 16) | (tmp[14] << 8) | tmp[13];
+ * mem->flags = tmp[0];
+ */
 }
 
+
 /*
  *  Add 32-bit fixed memory resource to resources list.
  */
@@ -650,17 +675,18 @@ static void isapnp_parse_fixed_mem32_resource(int size)
   unsigned char tmp[9];
 
   isapnp_peek(tmp, size);
-  /*
-   * mem = isapnp_alloc(sizeof(struct pnp_mem));
-   * if (!mem)
-   * return;
-   * mem->min = mem->max = (tmp[4] << 24) | (tmp[3] << 16) | (tmp[2] << 8) | tmp[1];
-   * mem->size = (tmp[8] << 24) | (tmp[7] << 16) | (tmp[6] << 8) | tmp[5];
-   * mem->align = 0;
-   * mem->flags = tmp[0];
-   */
+/*
+ * mem = isapnp_alloc(sizeof(struct pnp_mem));
+ * if (!mem)
+ * return;
+ * mem->min = mem->max = (tmp[4] << 24) | (tmp[3] << 16) | (tmp[2] << 8) | tmp[1];
+ * mem->size = (tmp[8] << 24) | (tmp[7] << 16) | (tmp[6] << 8) | tmp[5];
+ * mem->align = 0;
+ * mem->flags = tmp[0];
+ */
 }
 
+
 /*
  *  Parse card name for ISA PnP device.
  */
@@ -676,12 +702,13 @@ static string isapnp_parse_name(unsigned short &size)
   return hw::strip(buffer);
 }
 
+
 /*
  *  Parse resource map for logical device.
  */
 
 static int isapnp_create_device(hwNode & card,
-				unsigned short size)
+unsigned short size)
 {
   int number = 0, skip = 0, priority = 0, compat = 0;
   unsigned char type, tmp[17];
@@ -689,11 +716,11 @@ static int isapnp_create_device(hwNode & card,
 
   if ((dev = isapnp_parse_device(card, size, number++)) == NULL)
     return 1;
-  /*
-   * option = pnp_register_independent_option(dev);
-   * if (!option)
-   * return 1;
-   */
+/*
+ * option = pnp_register_independent_option(dev);
+ * if (!option)
+ * return 1;
+ */
 
   while (1)
   {
@@ -703,148 +730,150 @@ static int isapnp_create_device(hwNode & card,
       goto __skip;
     switch (type)
     {
-    case _STAG_LOGDEVID:
-      if (size >= 5 && size <= 6)
-      {
-	if ((dev = isapnp_parse_device(card, size, number++)) == NULL)
-	  return 1;
-	size = 0;
-	skip = 0;
-	/*
-	 * option = pnp_register_independent_option(dev);
-	 * if (!option)
-	 * return 1;
-	 * pnp_add_card_device(card,dev);
-	 */
-      }
-      else
-      {
-	skip = 1;
-      }
-      priority = 0;
-      compat = 0;
-      break;
-    case _STAG_COMPATDEVID:
-      if (size == 4 && compat < DEVICE_COUNT_COMPATIBLE)
-      {
-	isapnp_peek(tmp, 4);
-	if (dev->getClass() == hw::generic)
-	  dev->
-	    setClass(pnp_class
-		     (isapnp_parse_id
-		      ((tmp[1] << 8) | tmp[0], (tmp[3] << 8) | tmp[2])));
-	dev->
-	  addCapability(string(isapnp_parse_id
-			       ((tmp[1] << 8) | tmp[0],
-				(tmp[3] << 8) | tmp[2])) +
-			string("-compatible"));
-	compat++;
-	size = 0;
-      }
-      break;
-    case _STAG_IRQ:
-      if (size < 2 || size > 3)
-	goto __skip;
-      isapnp_parse_irq_resource(card, size);
-      size = 0;
-      break;
-    case _STAG_DMA:
-      if (size != 2)
-	goto __skip;
-      isapnp_parse_dma_resource(size);
-      size = 0;
-      break;
-    case _STAG_STARTDEP:
-      if (size > 1)
-	goto __skip;
-      //priority = 0x100 | PNP_RES_PRIORITY_ACCEPTABLE;
-      if (size > 0)
-      {
-	isapnp_peek(tmp, size);
-	priority = 0x100 | tmp[0];
-	size = 0;
-      }
-      /*
-       * option = pnp_register_dependent_option(dev,priority);
-       * if (!option)
-       * return 1;
-       */
-      break;
-    case _STAG_ENDDEP:
-      if (size != 0)
-	goto __skip;
-      priority = 0;
-      break;
-    case _STAG_IOPORT:
-      if (size != 7)
-	goto __skip;
-      isapnp_parse_port_resource(*dev, size);
-      size = 0;
-      break;
-    case _STAG_FIXEDIO:
-      if (size != 3)
-	goto __skip;
-      isapnp_parse_fixed_port_resource(size);
-      size = 0;
-      break;
-    case _STAG_VENDOR:
-      break;
-    case _LTAG_MEMRANGE:
-      if (size != 9)
-	goto __skip;
-      isapnp_parse_mem_resource(size);
-      size = 0;
-      break;
-    case _LTAG_ANSISTR:
-      dev->setProduct(isapnp_parse_name(size));
-      break;
-    case _LTAG_UNICODESTR:
-      /*
-       * silently ignore 
-       */
-      /*
-       * who use unicode for hardware identification? 
-       */
-      break;
-    case _LTAG_VENDOR:
-      break;
-    case _LTAG_MEM32RANGE:
-      if (size != 17)
-	goto __skip;
-      isapnp_parse_mem32_resource(size);
-      size = 0;
-      break;
-    case _LTAG_FIXEDMEM32RANGE:
-      if (size != 9)
-	goto __skip;
-      isapnp_parse_fixed_mem32_resource(size);
-      size = 0;
-      break;
-    case _STAG_END:
-      if (size > 0)
-	isapnp_skip_bytes(size);
-      return 1;
-    default:
-      break;
+      case _STAG_LOGDEVID:
+        if (size >= 5 && size <= 6)
+        {
+          if ((dev = isapnp_parse_device(card, size, number++)) == NULL)
+            return 1;
+          size = 0;
+          skip = 0;
+/*
+ * option = pnp_register_independent_option(dev);
+ * if (!option)
+ * return 1;
+ * pnp_add_card_device(card,dev);
+ */
+        }
+        else
+        {
+          skip = 1;
+        }
+        priority = 0;
+        compat = 0;
+        break;
+      case _STAG_COMPATDEVID:
+        if (size == 4 && compat < DEVICE_COUNT_COMPATIBLE)
+        {
+          isapnp_peek(tmp, 4);
+          if (dev->getClass() == hw::generic)
+            dev->
+              setClass(pnp_class
+              (isapnp_parse_id
+              ((tmp[1] << 8) | tmp[0], (tmp[3] << 8) | tmp[2])));
+          dev->
+            addCapability(string(isapnp_parse_id
+            ((tmp[1] << 8) | tmp[0],
+            (tmp[3] << 8) | tmp[2])) +
+            string("-compatible"));
+          compat++;
+          size = 0;
+        }
+        break;
+      case _STAG_IRQ:
+        if (size < 2 || size > 3)
+          goto __skip;
+        isapnp_parse_irq_resource(card, size);
+        size = 0;
+        break;
+      case _STAG_DMA:
+        if (size != 2)
+          goto __skip;
+        isapnp_parse_dma_resource(size);
+        size = 0;
+        break;
+      case _STAG_STARTDEP:
+        if (size > 1)
+          goto __skip;
+//priority = 0x100 | PNP_RES_PRIORITY_ACCEPTABLE;
+        if (size > 0)
+        {
+          isapnp_peek(tmp, size);
+          priority = 0x100 | tmp[0];
+          size = 0;
+        }
+/*
+ * option = pnp_register_dependent_option(dev,priority);
+ * if (!option)
+ * return 1;
+ */
+        break;
+      case _STAG_ENDDEP:
+        if (size != 0)
+          goto __skip;
+        priority = 0;
+        break;
+      case _STAG_IOPORT:
+        if (size != 7)
+          goto __skip;
+        isapnp_parse_port_resource(*dev, size);
+        size = 0;
+        break;
+      case _STAG_FIXEDIO:
+        if (size != 3)
+          goto __skip;
+        isapnp_parse_fixed_port_resource(size);
+        size = 0;
+        break;
+      case _STAG_VENDOR:
+        break;
+      case _LTAG_MEMRANGE:
+        if (size != 9)
+          goto __skip;
+        isapnp_parse_mem_resource(size);
+        size = 0;
+        break;
+      case _LTAG_ANSISTR:
+        dev->setProduct(isapnp_parse_name(size));
+        break;
+      case _LTAG_UNICODESTR:
+/*
+ * silently ignore
+ */
+/*
+ * who use unicode for hardware identification?
+ */
+        break;
+      case _LTAG_VENDOR:
+        break;
+      case _LTAG_MEM32RANGE:
+        if (size != 17)
+          goto __skip;
+        isapnp_parse_mem32_resource(size);
+        size = 0;
+        break;
+      case _LTAG_FIXEDMEM32RANGE:
+        if (size != 9)
+          goto __skip;
+        isapnp_parse_fixed_mem32_resource(size);
+        size = 0;
+        break;
+      case _STAG_END:
+        if (size > 0)
+          isapnp_skip_bytes(size);
+        return 1;
+      default:
+        break;
     }
-  __skip:
+    __skip:
     if (size > 0)
       isapnp_skip_bytes(size);
   }
   return 0;
 }
 
+
 static string bcd_version(unsigned char v,
-			  const char *separator = "")
+const char *separator = "")
 {
   char version[10];
 
   snprintf(version, sizeof(version), "%d%s%d", (v & 0xf0) >> 4, separator,
-	   v & 0x0f);
+    v & 0x0f);
 
   return string(version);
 }
 
+
 /*
  *  Parse resource map for ISA PnP card.
  */
@@ -860,45 +889,45 @@ static bool isapnp_parse_resource_map(hwNode & card)
       return false;
     switch (type)
     {
-    case _STAG_PNPVERNO:
-      if (size != 2)
-	goto __skip;
-      isapnp_peek(tmp, 2);
-      card.addCapability("pnp-" + bcd_version(tmp[0], "."));
-      card.setVersion(bcd_version(tmp[1]));
-      size = 0;
-      break;
-    case _STAG_LOGDEVID:
-      if (size >= 5 && size <= 6)
-      {
-	if (isapnp_create_device(card, size) == 1)
-	  return false;
-	size = 0;
-      }
-      break;
-    case _STAG_VENDOR:
-      break;
-    case _LTAG_ANSISTR:
-      card.setProduct(isapnp_parse_name(size));
-      break;
-    case _LTAG_UNICODESTR:
-      /*
-       * silently ignore 
-       */
-      /*
-       * who uses unicode for hardware identification? 
-       */
-      break;
-    case _LTAG_VENDOR:
-      break;
-    case _STAG_END:
-      if (size > 0)
-	isapnp_skip_bytes(size);
-      return true;
-    default:
-      break;
+      case _STAG_PNPVERNO:
+        if (size != 2)
+          goto __skip;
+        isapnp_peek(tmp, 2);
+        card.addCapability("pnp-" + bcd_version(tmp[0], "."));
+        card.setVersion(bcd_version(tmp[1]));
+        size = 0;
+        break;
+      case _STAG_LOGDEVID:
+        if (size >= 5 && size <= 6)
+        {
+          if (isapnp_create_device(card, size) == 1)
+            return false;
+          size = 0;
+        }
+        break;
+      case _STAG_VENDOR:
+        break;
+      case _LTAG_ANSISTR:
+        card.setProduct(isapnp_parse_name(size));
+        break;
+      case _LTAG_UNICODESTR:
+/*
+ * silently ignore
+ */
+/*
+ * who uses unicode for hardware identification?
+ */
+        break;
+      case _LTAG_VENDOR:
+        break;
+      case _STAG_END:
+        if (size > 0)
+          isapnp_skip_bytes(size);
+        return true;
+      default:
+        break;
     }
-  __skip:
+    __skip:
     if (size > 0)
       isapnp_skip_bytes(size);
   }
@@ -906,6 +935,7 @@ static bool isapnp_parse_resource_map(hwNode & card)
   return true;
 }
 
+
 /*
  *  Compute ISA PnP checksum for first eight bytes.
  */
@@ -922,33 +952,35 @@ static unsigned char isapnp_checksum(unsigned char *data)
     {
       bit = 0;
       if (b & (1 << j))
-	bit = 1;
+        bit = 1;
       checksum =
-	((((checksum ^ (checksum >> 1)) & 0x01) ^ bit) << 7) | (checksum >>
-								1);
+        ((((checksum ^ (checksum >> 1)) & 0x01) ^ bit) << 7) | (checksum >>
+        1);
     }
   }
   return checksum;
 }
 
+
 /*
  *  Parse EISA id for ISA PnP card.
  */
 
 static const char *isapnp_parse_card_id(unsigned short vendor,
-					unsigned short device)
+unsigned short device)
 {
   static char id[8];
 
   snprintf(id, sizeof(id), "%c%c%c%x%x%x%x",
-	   'A' + ((vendor >> 2) & 0x3f) - 1,
-	   'A' + (((vendor & 3) << 3) | ((vendor >> 13) & 7)) - 1,
-	   'A' + ((vendor >> 8) & 0x1f) - 1,
-	   (device >> 4) & 0x0f,
-	   device & 0x0f, (device >> 12) & 0x0f, (device >> 8) & 0x0f);
+    'A' + ((vendor >> 2) & 0x3f) - 1,
+    'A' + (((vendor & 3) << 3) | ((vendor >> 13) & 7)) - 1,
+    'A' + ((vendor >> 8) & 0x1f) - 1,
+    (device >> 4) & 0x0f,
+    device & 0x0f, (device >> 12) & 0x0f, (device >> 8) & 0x0f);
   return id;
 }
 
+
 /*
  *  Build device list for all present ISA PnP devices.
  */
@@ -968,24 +1000,25 @@ static int isapnp_build_device_list(hwNode & n)
     isapnp_peek(header, 9);
 
     hwNode card(isapnp_parse_card_id((header[1] << 8) | header[0],
-				       (header[3] << 8) | header[2]));
+      (header[3] << 8) | header[2]));
 
     checksum = isapnp_checksum(header);
-    /*
-     * Don't be strict on the checksum, here !
-     * e.g. 'SCM SwapBox Plug and Play' has header[8]==0 (should be: b7)
-     */
+/*
+ * Don't be strict on the checksum, here !
+ * e.g. 'SCM SwapBox Plug and Play' has header[8]==0 (should be: b7)
+ */
     if (header[8] == 0)
       ;
-    else if (checksum == 0x00 || checksum != header[8])	/* not valid CSN */
+                                                  /* not valid CSN */
+    else if (checksum == 0x00 || checksum != header[8])
       continue;
 
     card.setPhysId(csn);
     card.
       setVendor(vendorname
-		(isapnp_parse_card_id
-		 ((header[1] << 8) | header[0],
-		  (header[3] << 8) | header[2])));
+      (isapnp_parse_card_id
+      ((header[1] << 8) | header[0],
+      (header[3] << 8) | header[2])));
     card.addCapability("pnp");
     card.addCapability("isapnp");
     card.setBusInfo("isapnp@" + card.getPhysId());
@@ -1000,11 +1033,11 @@ static int isapnp_build_device_list(hwNode & n)
     }
     isapnp_checksum_value = 0x00;
     isapnp_parse_resource_map(card);
-    /*
-     * if (isapnp_checksum_value != 0x00)
-     * fprintf(stderr, "isapnp: checksum for device %i is not valid (0x%x)\n",
-     * csn, isapnp_checksum_value);
-     */
+/*
+ * if (isapnp_checksum_value != 0x00)
+ * fprintf(stderr, "isapnp: checksum for device %i is not valid (0x%x)\n",
+ * csn, isapnp_checksum_value);
+ */
 
     n.addChild(card);
   }
@@ -1012,11 +1045,11 @@ static int isapnp_build_device_list(hwNode & n)
   return 0;
 }
 
+
 static bool isabus(const hwNode & n)
 {
   return (n.getClass() == hw::bridge) && n.isCapable("isa");
 }
-
 #endif
 
 bool scan_isapnp(hwNode & n)
@@ -1039,7 +1072,6 @@ bool scan_isapnp(hwNode & n)
     isapnp_build_device_list(*isaroot);
   else
     isapnp_build_device_list(n);
-
 #endif
   return true;
 }
