@@ -26,6 +26,7 @@
 #include "device-tree.h"
 #include "pci.h"
 #include "pcmcia.h"
+#include "pcmcia-legacy.h"
 #include "ide.h"
 #include "scsi.h"
 #include "spd.h"
@@ -38,11 +39,13 @@
 #include "parisc.h"
 #include "cpufreq.h"
 #include "ideraid.h"
+#include "mounts.h"
+#include "smp.h"
 
 #include <unistd.h>
 #include <stdio.h>
 
-__ID("@(#) $Id: main.cc 1726 2007-02-13 21:38:11Z lyonel $");
+__ID("@(#) $Id: main.cc 1907 2007-10-13 22:23:16Z lyonel $");
 
 bool scan_system(hwNode & system)
 {
@@ -50,12 +53,15 @@ bool scan_system(hwNode & system)
 
   if (gethostname(hostname, sizeof(hostname)) == 0)
   {
-    hwNode computer(hostname,
+    hwNode computer(::enabled("output:sanitize")?"computer":hostname,
       hw::system);
 
     status("DMI");
     if (enabled("dmi"))
       scan_dmi(computer);
+    status("SMP");
+    if (enabled("smp"))
+      scan_smp(computer);
     status("PA-RISC");
     if (enabled("parisc"))
       scan_parisc(computer);
@@ -95,6 +101,9 @@ bool scan_system(hwNode & system)
     status("PCMCIA");
     if (enabled("pcmcia"))
       scan_pcmcia(computer);
+    status("PCMCIA");
+    if (enabled("pcmcia-legacy"))
+      scan_pcmcialegacy(computer);
     status("kernel device tree (sysfs)");
     if (enabled("sysfs"))
       scan_sysfs(computer);
@@ -109,6 +118,8 @@ bool scan_system(hwNode & system)
     status("SCSI");
     if (enabled("scsi"))
       scan_scsi(computer);
+    if (enabled("mounts"))
+      scan_mounts(computer);
     status("Network interfaces");
     if (enabled("network"))
       scan_network(computer);
