@@ -18,6 +18,7 @@
 #include "network.h"
 #include "osutils.h"
 #include "sysfs.h"
+#include "options.h"
 #include "heuristics.h"
 #include <sys/socket.h>
 #include <sys/ioctl.h>
@@ -34,7 +35,7 @@
 #include <sys/types.h>
 using namespace std;
 
-__ID("@(#) $Id: network.cc 1726 2007-02-13 21:38:11Z lyonel $");
+__ID("@(#) $Id: network.cc 1907 2007-10-13 22:23:16Z lyonel $");
 
 #ifndef ARPHRD_IEEE1394
 #define ARPHRD_IEEE1394 24
@@ -258,7 +259,7 @@ static void scan_ip(hwNode & interface)
     if (ioctl(fd, SIOCGIFADDR, &ifr) == 0)
     {
 // IP address is in ifr.ifr_addr
-      interface.setConfig("ip", print_ip((sockaddr_in *) (&ifr.ifr_addr)));
+      interface.setConfig("ip", ::enabled("output:sanitize")?REMOVED:print_ip((sockaddr_in *) (&ifr.ifr_addr)));
       strcpy(ifr.ifr_name, interface.getLogicalName().c_str());
       if ((interface.getConfig("point-to-point") == "yes")
         && (ioctl(fd, SIOCGIFDSTADDR, &ifr) == 0))
