@@ -1,5 +1,7 @@
 #include "version.h"
 #include "osutils.h"
+#include <sstream>
+#include <iomanip>
 #include <stack>
 #include <fcntl.h>
 #include <sys/stat.h>
@@ -11,11 +13,12 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <errno.h>
+#include <wchar.h>
 #ifndef MINOR
 #include <linux/kdev_t.h>
 #endif
 
-__ID("@(#) $Id: osutils.cc 1726 2007-02-13 21:38:11Z lyonel $");
+__ID("@(#) $Id: osutils.cc 1907 2007-10-13 22:23:16Z lyonel $");
 
 using namespace std;
 
@@ -284,8 +287,7 @@ string get_devid(const string & name)
 }
 
 
-bool samefile(const string & path1,
-const string & path2)
+bool samefile(const string & path1, const string & path2)
 {
   struct stat stat1;
   struct stat stat2;
@@ -406,6 +408,9 @@ string escape(const string & s)
       case '&':
         result += "&amp;";
         break;
+      case '"':
+        result += "&quot;";
+        break;
     default:
       result += s[i];
   }
@@ -414,7 +419,7 @@ string escape(const string & s)
 }
 
 
-unsigned short be_short(void * from)
+unsigned short be_short(const void * from)
 {
   __uint8_t *p = (__uint8_t*)from;
 
@@ -423,7 +428,7 @@ unsigned short be_short(void * from)
 }
 
 
-unsigned short le_short(void * from)
+unsigned short le_short(const void * from)
 {
   __uint8_t *p = (__uint8_t*)from;
 
@@ -432,7 +437,7 @@ unsigned short le_short(void * from)
 }
 
 
-unsigned long be_long(void * from)
+unsigned long be_long(const void * from)
 {
   __uint8_t *p = (__uint8_t*)from;
 
@@ -443,7 +448,7 @@ unsigned long be_long(void * from)
 }
 
 
-unsigned long le_long(void * from)
+unsigned long le_long(const void * from)
 {
   __uint8_t *p = (__uint8_t*)from;
 
@@ -455,7 +460,7 @@ unsigned long le_long(void * from)
 }
 
 
-unsigned long long be_longlong(void * from)
+unsigned long long be_longlong(const void * from)
 {
   __uint8_t *p = (__uint8_t*)from;
 
@@ -470,7 +475,7 @@ unsigned long long be_longlong(void * from)
 }
 
 
-unsigned long long le_longlong(void * from)
+unsigned long long le_longlong(const void * from)
 {
   __uint8_t *p = (__uint8_t*)from;
 
@@ -511,7 +516,6 @@ int open_dev(dev_t dev, const string & name)
   return -1;
 }                                                 /* open_dev */
 
-
 #define putchar(c) ((char)(c & 0xff))
 
 string utf8(wchar_t c)
@@ -543,3 +547,128 @@ string utf8(wchar_t c)
 
   return result;
 }
+
+string utf8(uint16_t * s, size_t length, bool forcelittleendian)
+{
+  string result = "";
+  size_t i;
+
+  for(i=0; (length<0) || (i<length); i++)
+    if(s[i])
+      result += utf8(forcelittleendian?le_short(s+i):s[i]);
+    else
+      break;	// NUL found
+
+  return result;
+}
+
+// U+FFFD replacement character
+#define REPLACEMENT  "\357\277\275"
+
+string utf8_sanitize(const string & s)
+{
+  unsigned int i = 0;
+  unsigned int remaining = 0;
+  string result = "";
+  string emit = "";
+  unsigned char c = 0;
+
+  while(i<s.length())
+  {
+    c = s[i];
+    switch(remaining)
+    {
+      case 3:
+      case 2:
+      case 1:
+        if((0x80<=c) && (c<=0xbf))
+        {
+          emit += s[i];
+          remaining--;
+        }
+        else		// invalid sequence (truncated)
+        {
+          emit = REPLACEMENT;
+          emit += s[i];
+          remaining = 0;
+        }
+        break;
+
+      case 0:
+        result += emit;
+        emit = "";
+
+        if(c<=0x7f)
+          emit = s[i];
+        else
+        if((0xc2<=c) && (c<=0xdf))	// start 2-byte sequence
+        {
+          remaining = 1;
+          emit = s[i];
+        }
+        else
+        if((0xe0<=c) && (c<=0xef))	// start 3-byte sequence
+        {
+          remaining = 2;
+          emit = s[i];
+        }
+        else
+        if((0xf0<=c) && (c<=0xf4))	// start 4-byte sequence
+        {
+          remaining = 3;
+          emit = s[i];
+        }
+        else
+          emit = REPLACEMENT;	// invalid character
+
+        break;
+    }
+
+    i++;
+  }
+
+  if(remaining == 0)
+    result += emit;
+
+  return result;
+}
+
+string decimalkilos(unsigned long long value)
+{
+  const char *prefixes = "KMGTPEZY";
+  unsigned int i = 0;
+  ostringstream out;
+
+  while ((i <= strlen(prefixes)) && ((value > 10000) || (value % 1000 == 0)))
+  {
+    value = value / 1000;
+    i++;
+  }
+
+  out << value;
+  if ((i > 0) && (i <= strlen(prefixes)))
+    out << prefixes[i - 1];
+
+  return out.str();
+}
+
+
+string kilobytes(unsigned long long value)
+{
+  const char *prefixes = "KMGTPEZY";
+  unsigned int i = 0;
+  ostringstream out;
+
+  while ((i <= strlen(prefixes)) && ((value > 10240) || (value % 1024 == 0)))
+  {
+    value = value >> 10;
+    i++;
+  }
+
+  out << value;
+  if ((i > 0) && (i <= strlen(prefixes)))
+    out << prefixes[i - 1];
+  out << "iB";
+
+  return out.str();
+}
