@@ -16,8 +16,8 @@ std::string realpath(const std::string & path);
 bool loadfile(const std::string & file, std::vector < std::string > &lines);
 
 size_t splitlines(const std::string & s,
-		std::vector < std::string > &lines,
-		char separator = '\n');
+std::vector < std::string > &lines,
+char separator = '\n');
 std::string get_string(const std::string & path, const std::string & def = "");
 
 std::string find_deventry(mode_t mode, dev_t device);
@@ -25,6 +25,10 @@ std::string get_devid(const std::string &);
 
 std::string uppercase(const std::string &);
 std::string lowercase(const std::string &);
+std::string tostring(unsigned long long);
+std::string utf8(wchar_t);
+std::string utf8_sanitize(const std::string &);
+std::string utf8(uint16_t *s, size_t length = -1, bool forcelittleendian = false);
 
 std::string spaces(unsigned int count, const std::string & space = " ");
 
@@ -32,18 +36,20 @@ std::string escape(const std::string &);
 
 std::string join(const std::string &, const std::string &, const std::string &);
 
+std::string decimalkilos(unsigned long long value);
+std::string kilobytes(unsigned long long value);
+
 bool matches(const std::string & s, const std::string & pattern, int cflags=0);
 
 int selectdir(const struct dirent *d);
 int selectlink(const struct dirent *d);
 
-unsigned short be_short(void *);
-unsigned short le_short(void *);
-unsigned long be_long(void *);
-unsigned long le_long(void *);
-unsigned long long be_longlong(void *);
-unsigned long long le_longlong(void *);
+unsigned short be_short(const void *);
+unsigned short le_short(const void *);
+unsigned long be_long(const void *);
+unsigned long le_long(const void *);
+unsigned long long be_longlong(const void *);
+unsigned long long le_longlong(const void *);
 
 int open_dev(dev_t dev, const std::string & name="");
-
 #endif
