@@ -21,7 +21,7 @@
 #include <string>
 #include <map>
 
-__ID("@(#) $Id: scsi.cc 1726 2007-02-13 21:38:11Z lyonel $");
+__ID("@(#) $Id: scsi.cc 1907 2007-10-13 22:23:16Z lyonel $");
 
 #define SG_X "/dev/sg%d"
 #define SG_MAJOR 21
@@ -529,16 +529,21 @@ hwNode & node)
 
     node.setCapacity(heads * cyl * sectors * sectsize);
 
-    if (rpm / 10000 == 1)
-      node.addCapability("10000rpm", "10000 rotations per minute");
+    if (rpm / 15000 == 1)
+      node.addCapability("15000rpm", "15000 rotations per minute");
     else
     {
-      if (rpm / 7200 == 1)
-        node.addCapability("7200rpm", "7200 rotations per minute");
+      if (rpm / 10000 == 1)
+        node.addCapability("10000rpm", "10000 rotations per minute");
       else
       {
-        if (rpm / 5400 == 1)
-          node.addCapability("5400rpm", "5400 rotations per minute");
+        if (rpm / 7200 == 1)
+          node.addCapability("7200rpm", "7200 rotations per minute");
+        else
+        {
+          if (rpm / 5400 == 1)
+            node.addCapability("5400rpm", "5400 rotations per minute");
+        }
       }
     }
   }
@@ -725,15 +730,24 @@ hwNode & n)
       break;
   }
 
-  device.setDescription("SCSI " + string(scsi_type(m_id.scsi_type)));
+  device.setDescription(string(scsi_type(m_id.scsi_type)));
   device.setHandle(scsi_handle(m_id.host_no,
     m_id.channel, m_id.scsi_id, m_id.lun));
   device.setBusInfo(scsi_businfo(m_id.host_no,
     m_id.channel, m_id.scsi_id, m_id.lun));
   device.setPhysId(m_id.channel, m_id.scsi_id, m_id.lun);
-  device.addHint("bus.icon", string("scsi"));
   find_logicalname(device);
   do_inquiry(fd, device);
+  if(device.getVendor() == "ATA")
+  {
+    device.setDescription("ATA " + device.getDescription());
+    device.setVendor("");
+  }
+  else
+  {
+    device.setDescription("SCSI " + device.getDescription());
+    device.addHint("bus.icon", string("scsi"));
+  }
   if ((m_id.scsi_type == 4) || (m_id.scsi_type == 5))
     scan_cdrom(device);
   if ((m_id.scsi_type == 0) || (m_id.scsi_type == 7) || (m_id.scsi_type == 14))
