@@ -1,15 +1,28 @@
 #include "engine.h"
 #include "hw.h"
 #include "main.h"
+#include "print-gui.h"
 #include "print.h"
+#include "osutils.h"
+#include "options.h"
 
-static char *id = "@(#) $Id: engine.cc 1726 2007-02-13 21:38:11Z lyonel $";
+#include <iostream>
+#include <fstream>
+#include <sys/utsname.h>
+#include <libgen.h>
+
+static char *id = "@(#) $Id: engine.cc 1907 2007-10-13 22:23:16Z lyonel $";
 
 extern "C"
 {
 #include "support.h"
 };
 
+#define AUTOMATIC "automatic"
+#define LSHW_XML "lshw XML format"
+#define PLAIN_TEXT "plain text document"
+#define HTML "HTML document"
+
 #define YIELD()  while(gtk_events_pending()) gtk_main_iteration()
 
 static hwNode container("container", hw::generic);
@@ -204,7 +217,7 @@ static void display(GtkWidget * mainwindow)
   {
     create_tags(buffer);
 
-    string hwpath = printhwpath(*displayed, container);
+    string hwpath = gethwpath(*displayed, container);
     printmarkup(*displayed, GTK_TEXT_VIEW(description), hwpath);
   }
 }
@@ -233,10 +246,15 @@ void refresh(GtkWidget *mainwindow)
   static bool lock = false;
   GtkWidget * description = lookup_widget(mainwindow, "description");
   GtkWidget * go_up_button = lookup_widget(mainwindow, "upbutton");
+  GtkWidget * save_button = lookup_widget(mainwindow, "savebutton");
+  GtkWidget * menu = lookup_widget(mainwindow, "menu");
+  GtkWidget * save_menuitem = lookup_widget(menu, "save");
 
   if(lock) return;
 
   lock = true;
+  gtk_widget_set_sensitive(save_button, FALSE);
+  gtk_widget_set_sensitive(save_menuitem, FALSE);
 
   GtkWidget * list1 = lookup_widget(mainwindow, "treeview1");
   GtkWidget * list2 = lookup_widget(mainwindow, "treeview2");
@@ -249,13 +267,16 @@ void refresh(GtkWidget *mainwindow)
   displayed = NULL;
   gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(description)), "scanning...", -1);
 
+  disable("output:sanitize");
   container = hwNode("container", hw::generic);
   status("Scanning...");
   scan_system(computer);
   status(NULL);
   displayed = container.addChild(computer);
 
-  gtk_widget_set_sensitive(go_up_button, 0);
+  gtk_widget_set_sensitive(go_up_button, FALSE);
+  gtk_widget_set_sensitive(save_button, TRUE);
+  gtk_widget_set_sensitive(save_menuitem, TRUE);
 
   selected1 = NULL;
   selected2 = NULL;
@@ -402,3 +423,179 @@ void go_back(GtkWidget *mainwindow)
 
   display(mainwindow);
 }
+
+static const char *guess_format(char *s)
+{
+  char *dot = strrchr(s, '.');
+
+  if(!dot)
+    return LSHW_XML;
+
+  if(!strcasecmp(".html", dot) || !strcasecmp(".htm", dot))
+    return HTML;
+
+  if(!strcasecmp(".text", dot) || !strcasecmp(".txt", dot))
+    return PLAIN_TEXT;
+
+  return LSHW_XML;
+}
+
+static char *fix_filename(char *s, char *extension)
+{
+  char *dot = strrchr(s, '.');
+
+  if(dot)
+    return s;
+
+  s = (char*)realloc(s, strlen(s) + 1 + strlen(extension) + 1);
+  strcat(s, ".");
+  strcat(s, extension);
+
+  return s;
+}
+
+static void redirect_cout(std::ofstream &out, bool enable = true)
+{
+  static std::streambuf* old_cout;
+  
+  if(enable)
+  {
+    old_cout = cout.rdbuf();
+    cout.rdbuf(out.rdbuf());
+  }
+  else
+    cout.rdbuf(old_cout);
+}
+
+void save_as(GtkWidget *mainwindow)
+{
+  struct utsname buf;
+  GtkWidget *dialog = NULL;
+  GtkWidget *sanitize = NULL;
+  GtkFileFilter *filter = NULL;
+  bool proceed = true;
+  hwNode *computer = container.getChild(0);
+
+  if(!computer)		// nothing to save
+    return;
+
+  dialog = gtk_file_chooser_dialog_new ("Save hardware configuration",
+				      GTK_WINDOW(mainwindow),
+				      GTK_FILE_CHOOSER_ACTION_SAVE,
+				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+				      NULL);
+  //gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
+  sanitize = gtk_check_button_new_with_label("Anonymize output");
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sanitize), enabled("output:sanitize")?TRUE:FALSE);
+  gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(dialog), sanitize);
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name(filter, AUTOMATIC);
+  gtk_file_filter_add_pattern(filter, "*");
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name(filter, LSHW_XML);
+  gtk_file_filter_add_pattern(filter, "*.lshw");
+  gtk_file_filter_add_pattern(filter, "*.xml");
+  gtk_file_filter_add_mime_type(filter, "text/xml");
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name(filter, HTML);
+  gtk_file_filter_add_pattern(filter, "*.html");
+  gtk_file_filter_add_pattern(filter, "*.htm");
+  gtk_file_filter_add_mime_type(filter, "text/html");
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_set_name(filter, PLAIN_TEXT);
+  gtk_file_filter_add_pattern(filter, "*.text");
+  gtk_file_filter_add_pattern(filter, "*.txt");
+  gtk_file_filter_add_mime_type(filter, "text/plain");
+  gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(dialog), filter);
+
+  if(uname(&buf)==0)
+    gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), buf.nodename);
+
+  if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+  {
+    char *filename;
+
+    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sanitize)))
+      enable("output:sanitize");
+    else
+      disable("output:sanitize");
+
+    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+    filter = gtk_file_chooser_get_filter(GTK_FILE_CHOOSER(dialog));
+    if(filename && filter)
+    {
+      const gchar *filtername = gtk_file_filter_get_name(filter);
+
+      if(strcmp(filtername, AUTOMATIC)==0)
+        filtername = guess_format(filename);
+
+      if(!exists(filename))		// creating a new file
+      {
+        if(strcmp(filtername, LSHW_XML)==0)
+          filename = fix_filename(filename, "lshw");
+        else
+        if(strcmp(filtername, HTML)==0)
+          filename = fix_filename(filename, "html");
+        else
+        if(strcmp(filtername, PLAIN_TEXT)==0)
+          filename = fix_filename(filename, "txt");
+      }
+
+      if(exists(filename))		// existing file
+      {
+        char * buffer1 = g_strdup(filename);
+        char * buffer2 = g_strdup(filename);
+
+        GtkWidget *dialog = gtk_message_dialog_new_with_markup (GTK_WINDOW(mainwindow),
+                                  GTK_DIALOG_DESTROY_WITH_PARENT,
+                                  GTK_MESSAGE_WARNING,
+                                  GTK_BUTTONS_NONE,
+                                  "A file named <i><tt>%s</tt></i> already exists in folder <tt>%s</tt>.\n\nDo you want to overwrite it?",
+                                  basename(buffer1), dirname(buffer2));
+        gtk_dialog_add_buttons(GTK_DIALOG(dialog), 
+				  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				  "Overwrite", GTK_RESPONSE_ACCEPT,
+                                  NULL);
+        proceed = (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT);
+        gtk_widget_destroy (dialog);
+        g_free(buffer1);
+        g_free(buffer2);
+      }
+
+      if(proceed)
+      {
+        if(strcmp(filtername, LSHW_XML)==0)
+        {
+          std::ofstream out(filename);
+          redirect_cout(out);
+          cout << computer->asXML();
+          redirect_cout(out, false);
+        }
+        else
+        if(strcmp(filtername, HTML)==0)
+        {
+          std::ofstream out(filename);
+          redirect_cout(out);
+          print(*computer, true);
+          redirect_cout(out, false);
+        }
+        else
+        if(strcmp(filtername, PLAIN_TEXT)==0)
+        {
+          std::ofstream out(filename);
+          redirect_cout(out);
+          print(*computer, false);
+          redirect_cout(out, false);
+        }
+      }
+      g_free (filename);
+    }
+  }
+
+  gtk_widget_destroy (dialog);
+}
