@@ -10,7 +10,7 @@
 #include <stdio.h>
 #include <iostream>
 
-__ID("@(#) $Id: lshw.cc 1726 2007-02-13 21:38:11Z lyonel $");
+__ID("@(#) $Id: lshw.cc 1907 2007-10-13 22:23:16Z lyonel $");
 
 void usage(const char *progname)
 {
@@ -35,6 +35,7 @@ void usage(const char *progname)
   fprintf(stderr,
     "\t-enable TEST    enable a test (like pci, isapnp, cpuid, etc. )\n");
   fprintf(stderr, "\t-quiet          don't display status\n");
+  fprintf(stderr, "\t-sanitize       sanitize output (remove sensitive information like serial numbers, etc.)\n");
   fprintf(stderr, "\n");
 }
 
@@ -43,7 +44,7 @@ void status(const char *message)
 {
   static size_t lastlen = 0;
 
-  if(enabled("quiet") || disabled("verbose"))
+  if(enabled("output:quiet") || disabled("output:verbose"))
     return;
 
   if (isatty(2))
@@ -61,14 +62,15 @@ void status(const char *message)
 int main(int argc,
 char **argv)
 {
-  bool htmloutput = false;
-  bool xmloutput = false;
-  bool hwpath = false;
-  bool businfo = false;
-  bool X = false;
-
   disable("isapnp");
-  disable("quiet");
+
+  disable("output:xml");
+  disable("output:html");
+  disable("output:hwpath");
+  disable("output:businfo");
+  disable("output:X");
+  disable("output:quiet");
+  disable("output:sanitize");
 
 // define some aliases for nodes classes
   alias("disc", "disk");
@@ -85,13 +87,22 @@ char **argv)
     exit(1);
   }
 
-  if (argc == 2)
+  while (argc >= 2)
   {
+    bool validoption = false;
+
     if (strcmp(argv[1], "-version") == 0)
     {
+      const char *latest = checkupdates();
       printf("%s\n", getpackageversion());
+      if(latest)
+      {
+        if(strcmp(latest, getpackageversion()) != 0)
+          fprintf(stderr, "the latest version is %s\n", latest);
+      }
       exit(0);
     }
+
     if (strcmp(argv[1], "-help") == 0 || strcmp(argv[1], "--help") == 0)
     {
       usage(argv[0]);
@@ -100,40 +111,65 @@ char **argv)
 
     if (strcmp(argv[1], "-verbose") == 0)
     {
-      disable("quiet");
-      enable("verbose");
-      argc--;
+      disable("output:quiet");
+      enable("output:verbose");
+      validoption = true;
     }
 
     if (strcmp(argv[1], "-quiet") == 0)
     {
-      disable("verbose");
-      enable("quiet");
-      argc--;
+      disable("output:verbose");
+      enable("output:quiet");
+      validoption = true;
     }
 
     if (strcmp(argv[1], "-xml") == 0)
-      xmloutput = true;
+    {
+      enable("output:xml");
+      validoption = true;
+    }
 
     if (strcmp(argv[1], "-html") == 0)
-      htmloutput = true;
+    {
+      enable("output:html");
+      validoption = true;
+    }
 
     if (strcmp(argv[1], "-short") == 0)
-      hwpath = true;
+    {
+      enable("output:hwpath");
+      validoption = true;
+    }
 
     if (strcmp(argv[1], "-businfo") == 0)
-      businfo = true;
+    {
+      enable("output:businfo");
+      validoption = true;
+    }
 
     if (strcmp(argv[1], "-X") == 0)
-      X = true;
+    {
+      enable("output:X");
+      validoption = true;
+    }
 
-    if ((argc>1) && !xmloutput && !htmloutput && !hwpath && !businfo && !X)
+    if ((strcmp(argv[1], "-sanitize") == 0) ||
+       (strcmp(argv[1], "-sanitise") == 0))
+    {
+      enable("output:sanitize");
+      validoption = true;
+    }
+
+    if(validoption)
+    {	/* shift */
+      memmove(argv+1, argv+2, (argc-1)*(sizeof(argv[0])));
+      argc--;
+    }
+    else
     {
       usage(argv[0]);
       exit(1);
     }
-    else
-      argc--;
   }
 
   if (argc >= 2)
@@ -142,7 +178,7 @@ char **argv)
     exit(1);
   }
 
-  if(X) execl(SBINDIR"/gtk-lshw", SBINDIR"/gtk-lshw", NULL);
+  if(enabled("output:X")) execl(SBINDIR"/gtk-lshw", SBINDIR"/gtk-lshw", NULL);
 
   if (geteuid() != 0)
   {
@@ -155,17 +191,17 @@ char **argv)
 
     scan_system(computer);
 
-    if (hwpath)
+    if (enabled("output:hwpath"))
       printhwpath(computer);
     else
-    if (businfo)
+    if (enabled("output:businfo"))
       printbusinfo(computer);
     else
     {
-      if (xmloutput)
+      if (enabled("output:xml"))
         cout << computer.asXML();
       else
-        print(computer, htmloutput);
+        print(computer, enabled("output:html"));
     }
   }
 
